"""
A simple utility that collects the entire ledger for an exchange account for a given
period and outputs it in .csv format. Very useful for year-end accounting.
"""

import pytz
import sys

# TODO: this script should do it's own imports, not rely on the console.
from gryphon.execution.console import *

import gryphon.lib.exchange.exchange_factory as exchange_factory
from gryphon.lib.time_parsing import parse


# This dictionary should be a list of [blockchain address]:exchange_name pairs, mapping
# the addresses which you use to move crypto-assets between exchanges to the exchange
# names. It's useful to have crypto deposits/withdrawals identified by where the asset
# was going in the ledger.
address_map = {
    u'1234567890bcdefghijklmnopqrstuvwxy': u'BITSTAMP',
}


def get_ledger_table_for_time(exchange_name, start_time, end_time, currency=None):
    if currency:
        currency = currency.upper()

    exchange_name = exchange_name.capitalize()
    exchange = exchange_factory.make_exchange_from_key(exchange_name)

    exchange_data = exchange_factory.make_exchange_data_from_key(
        exchange_name,
        db,
    )

    fiat_currency = exchange.currency

    ledger = exchange_data.ledger(
        start_time=start_time,
        end_time=end_time,
        currency=currency,
    )

    # ledger is empty for this time period.
    if len(ledger) == 0:
        return fiat_currency, currency, []

    # get the exchange balance from before these ledger entries started
    oldest_entry = ledger[-1]
    oldest_time = oldest_entry.time_created

    # There can be multiple entries at the same time
    entries_at_oldest_time = []

    for entry in ledger:
        if isinstance(entry, Trade) and entry.time_created == oldest_time:
            entries_at_oldest_time.append(entry)
        elif isinstance(entry, Transaction) and entry.time_completed == oldest_time:
            entries_at_oldest_time.append(entry)

    oldest_time = pytz.utc.localize(oldest_time)
    balance_at_oldest_time = exchange_data.ledger_balance(end_time=oldest_time)

    # balance_at_oldest_time includes all the entries_at_oldest_time
    # so we need to remove them to get the pre-ledger balance
    pre_ledger_balance = balance_at_oldest_time

    for entry in entries_at_oldest_time:
        pre_ledger_balance -= entry.position


    starting_ledger_balance = exchange_data.ledger_balance(end_time=start_time)
    ledger_diff = exchange_data.ledger_balance(
        start_time=start_time,
        end_time=end_time,
    )
    ending_ledger_balance = starting_ledger_balance + ledger_diff

    ledger_table = []

    for entry in ledger:
        if isinstance(entry, Trade):
            table_entries = table_entries_from_trade(entry)
        elif isinstance(entry, Transaction):
            table_entries = table_entries_from_transaction(entry)

        ledger_table += table_entries

    #ledger_table = self.filter_to_currency(ledger_table, currency)

    current_balance = starting_ledger_balance

    for table_entry in reversed(ledger_table):
        if 'credit' in table_entry:
            current_balance += table_entry['credit']
        if 'debit' in table_entry:
            current_balance -= table_entry['debit']

        table_entry['balance'] = current_balance

    return fiat_currency, currency, ledger_table


def table_entries_from_trade(trade):
    entries = []

    if trade.trade_type == Trade.BID:
        desc = 'Trade #%s: Buy %s for %s' % (trade.exchange_trade_id, trade.volume, trade.price)
        entries.append({
            'credit': trade.volume,
            'description': desc,
            'type': 'Trade - BUY'
        })
        entries.append({'debit': trade.price, 'description': desc, 'type': 'Trade - BUY'})

    elif trade.trade_type == Trade.ASK:
        desc = 'Trade #%s: Sell %s for %s' % (
            trade.exchange_trade_id,
            trade.volume,
            trade.price,
        )
        entries.append({
            'credit': trade.price,
            'description': desc,
            'type': 'Trade - SELL'
        })
        entries.append({'debit': trade.volume, 'description': desc, 'type': 'Trade - SELL'})

    if trade.fee:
        entries.append({
            'debit': trade.fee,
            'description': 'Trade #%s: Fee' % trade.exchange_trade_id,
            'type': 'Trade - FEE',
        })

    date = trade.time_created
    for entry in entries:
        entry['date'] = date
        entry['details'] = ''

    return entries


def table_entries_from_transaction(transaction):
    entries = []

    if 'drift' in transaction.transaction_details:
        # Skipping Drift Transactions
        return entries

    if transaction.transaction_type == Transaction.WITHDRAWL:
        deposit_address = transaction.transaction_details.get('deposit_address')
        destination = get_exchange_for_address(deposit_address)
        desc = 'Withdrawal to %s' % destination

        entries.append({
            'debit': transaction.amount,
            'description': desc,
            'type': 'Withdrawal - %s' % destination,
        })

    elif transaction.transaction_type == Transaction.DEPOSIT:
        desc = 'Deposit'
        entries.append({
            'credit': transaction.amount,
            'description': desc,
            'type': 'Deposit',
        })

    if transaction.fee:
        entries.append({
            'debit': transaction.fee,
            'description': 'Transaction Fee',
            'type': 'Transaction Fee',
        })

    date = transaction.time_completed
    if not date:
        date = transaction.time_created
    for entry in entries:
        entry['date'] = date
        entry['details'] = ''.join([
            '%s:%s ' % (k, v)
            for k, v in transaction.transaction_details.iteritems()
            if k in ['external_transaction_id', 'notes'] and v not in ['xxx']
        ])

    return entries


def get_exchange_for_address(address):

    if address and address in address_map:
        return address_map[address]
    else:
        return 'Non Exchange Transfer or Unknown Address'


def export_ledger_for_exchange(exchange_name, start_time, end_time):
    exchange_obj = exchange_factory.make_exchange_from_key(exchange_name)

    lines = 'Description, %s Credit, %s Debit, BTC Credit, BTC Debit, BTC Balance, %s Balance, Date (UTC)\n' % (exchange_obj.currency, exchange_obj.currency, exchange_obj.currency)

    _, _, ledger_table = get_ledger_table_for_time(exchange_name, start_time, end_time)

    for table_entry in ledger_table:
        fiat_debit = ''
        btc_debit = ''
        fiat_credit = ''
        btc_credit = ''

        if 'debit' in table_entry:
            if table_entry['debit'].currency == 'BTC':
                btc_debit = table_entry['debit'].amount
            else:
                fiat_debit = table_entry['debit'].amount

        if 'credit' in table_entry:
            if table_entry['credit'].currency == 'BTC':
                btc_credit = table_entry['credit'].amount
            else:
                fiat_credit = table_entry['credit'].amount

        btc_balance = table_entry['balance']['BTC'].amount

        fiat_balance = table_entry['balance'][exchange_obj.currency].amount

        date = table_entry.get('date')

        lines += '"%s",%s,%s,"%s","%s","%s","%s",%s\n' % (
            table_entry.get('description', ''),
            fiat_credit,
            fiat_debit,
            btc_credit,
            btc_debit,
            btc_balance,
            fiat_balance,
            date,
        )
    
    filename = '%s_ledger_%s_%s.csv' % (
        exchange_name.lower(),
        start_time.strftime('%Y-%m-%d'),
        end_time.strftime('%Y-%m-%d'),
    )

    f = open(filename, 'w')
    f.write(lines)
    f.close()


def main(script_arguments, execute):
    exchange_name = script_arguments['exchange'].upper()
    start_str = script_arguments['start']
    end_str = script_arguments['end']

    start_time = parse(start_str).datetime
    end_time = parse(end_str).datetime

    export_ledger_for_exchange(exchange_name, start_time, end_time)




# -*- coding: utf-8 -*-
from collections import OrderedDict
import hashlib

import cdecimal
from cdecimal import Decimal
from delorean import Delorean, parse

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

logger = get_logger(__name__)


class OKCoinBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(OKCoinBTCUSDExchange, self).__init__(session)

        self.name = u'OKCOIN_BTC_USD'
        self.friendly_name = u'OKCoin BTC-USD'
        self.currency = u'USD'
        self.volume_currency = 'BTC'
        self.base_url = 'https://www.okcoin.com/api/v1'
        self.bid_string = 'buy'
        self.ask_string = 'sell'

        # Codes, accurate for requests but not for responses. Responses number
        # -1 through 2.
        self.PARAM_ORDER_STATUS_UNFILLED = 0
        self.PARAM_ORDER_STATUS_FILLED = 1

        # Codes these are the statuses that the endpoints give back.
        self.ORDER_STATUS_CANCELLED = -1
        self.ORDER_STATUS_OPEN = 0
        self.ORDER_STATUS_PARTIALLY_FILLED = 1
        self.ORDER_STATUS_FULLY_FILLED = 2

        self.errors = {
            10000: 'Required field, can not be null',
            10001: 'Request frequency too high',
            10002: 'System error',
            10003: 'Not in reqest list, please try again later',
            10004: 'IP not allowed to access the resource',
            10005: '\'secretKey\' does not exist',
            10006: '\'partner\' does not exist',
            10007: 'Signature does not match',
            10008: 'Illegal parameter',
            10009: 'Order does not exist',
            10010: 'Insufficient funds',
            10011: 'Amount too low',
            10012: 'Only btc_usd ltc_usd supported',
            10013: 'Only support https request',
            10014: 'Order price must be between 0 and 1,000,000',
            10015: 'Order price differs from current market price too much',
            10016: 'Insufficient coins balance',
            10017: 'API authorization error',
            10026: 'Loan (including reserved loan) and margin cannot be withdrawn',
            10027: 'Cannot withdraw within 24 hrs of authentication information modification',
            10028: 'Withdrawal amount exceeds daily limit',
            10029: 'Account has unpaid loan, please cancel/pay off the loan before withdraw',
            10031: 'Deposits can only be withdrawn after 6 confirmations',
            10032: 'Please enabled phone/google authenticator',
            10033: 'Fee higher than maximum network transaction fee',
            10034: 'Fee lower than minimum network transaction fee',
            10035: 'Insufficient BTC/LTC',
            10036: 'Withdrawal amount too low',
            10037: 'Trade password not set',
            10040: 'Withdrawal cancellation fails',
            10041: 'Withdrawal address not approved',
            10042: 'Admin password error',
            10100: 'User account frozen',
        }

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.001')
        self.limit_order_fee = Decimal('0.002')
        self.fee = self.market_order_fee
        self.withdrawal_fee = Money('0.001', 'BTC')
        self.use_cached_orderbook = False
        self.max_tick_speed = 1

        if configuration:
            self.configure(configuration)

    def resp(self, req):
        response = super(OKCoinBTCUSDExchange, self).resp(req)

        error_code = response.get('error_code', None)

        if error_code:
            if error_code == 10009:
                raise exceptions.CancelOrderNotFoundError()
            elif error_code in [10016, 10010]:
                raise exceptions.InsufficientFundsError()
            else:
                raise exceptions.ExchangeAPIErrorException(
                    self,
                    self.errors[error_code],
                )

        return response

    def round(self, m):
        if m.currency == 'USD':
            return m.round_to_decimal_places(
                4,
                rounding=cdecimal.ROUND_HALF_EVEN,
            )
        elif m.currency == 'BTC':
            return m.round_to_decimal_places(
                8,
                rounding=cdecimal.ROUND_HALF_EVEN,
            )

    def ticker_req(self, verify=True):
        return self.req('get', '/ticker.do', no_auth=True, verify=verify)


    def ticker_resp(self, req):
        response = self.resp(req)

        response = response['ticker']

        return {
            'high': Money(response['high'], 'USD'),
            'low': Money(response['low'], 'USD'),
            'last': Money(response['last'], 'USD'),
            'bid': Money(response['buy'], 'USD'),
            'ask': Money(response['sell'], 'USD'),
            'volume': Money(response['vol'], 'BTC')
        }


    def all_transactions(self, page=0, unfilled_orders=None):
        req = self.all_transactions_req(
            page=page, 
            unfilled_orders=unfilled_orders
        )

        return self.all_transactions_resp(req)


    def all_transactions_req(self, page=0, unfilled_orders=None):
        # The unfilled orders parameter allows us to query for filled orders with
        # status=1 and unfilled orders with status=0.

        status = (self.PARAM_ORDER_STATUS_UNFILLED if unfilled_orders
                      else self.PARAM_ORDER_STATUS_FILLED)

        payload = {
            'symbol': 'btc_usd',
            'status': status,
            'current_page': page,
            'page_length': 200,
        }

        return self.req('post', '/order_history.do', data=payload)

    def all_transactions_resp(self, req):
        return self.resp(req)

    ###### Common Exchange Methods ######

    def load_creds(self):
        try:
            self.api_key
            self.secret
            self.partner_id
        except AttributeError:
            self.api_key = self._load_env('OKCOIN_BTC_USD_API_KEY')
            self.secret = self._load_env('OKCOIN_BTC_USD_API_SECRET')
            self.partner_id = self._load_env('OKCOIN_BTC_USD_PARTNER_ID')

    def auth_request(self, req_method, url, request_args):
        self.load_creds()

        try:
            payload = request_args['data']
        except KeyError:
            payload = request_args['data'] = {}
       
        payload.update({
            'api_key': self.api_key,
        })

        signature = self.get_okcoin_signature(payload, self.secret),

        payload.update({
            'sign': signature[0],
        })

    def get_okcoin_signature(self, params, secretKey):
        """
        Helper function, written by OKCoin, for building the auth signature.
        """
        sign = ''

        for key in sorted(params.keys()):
            sign += key + '=' + str(params[key]) + '&'

        return hashlib.md5(sign + 'secret_key=' + secretKey).hexdigest().upper()


    def balance_req(self):
        return self.req('post', '/userinfo.do')


    def balance_resp(self, req):
        response = self.resp(req)

        try:
            balances = response['info']['funds']

            btc_available = Money(balances['free']['btc'], 'BTC')
            usd_available = Money(balances['free']['usd'], 'USD')
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'Balance missing expected keys',
            )

        balance = Balance()
        balance['BTC'] = btc_available
        balance['USD'] = usd_available

        return balance

    def _get_order_book_req(self, verify=True):
        return self.req('get', '/depth.do', no_auth=True, verify=verify)
            
    def create_trade_req(self, mode, volume, price, is_market_order=False):
        volume = self.round(volume)
        price = self.round(price)

        trade_type = self.from_const(mode)

        try:
            payload = {
                'symbol': 'btc_usd',
                'type': trade_type,
                'amount': volume.amount,
                'price': price.amount,
            }

        except AttributeError:
            raise TypeError('volume and price must be Money objects')

        return self.req('post', '/trade.do', data=payload)

    def create_trade_resp(self, req):
        response = self.resp(req)

        try:
            return {
                'success': True, 
                'order_id': str(response['order_id'])
            }

        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order id',
            )


    def order_fee(self, order_id):
        req = self.order_fee_req(order_id)
        return self.order_fee_resp(req)

    def order_fee_req(self, order_id):
        payload = {
            'order_id': order_id,
            'symbol': 'btc_usd',
        }

        return self.req('post', '/order_fee.do', data=payload)

    def order_fee_resp(self, req):
        try:
            response = self.resp(req)
        except CancelOrderNotFoundError:
            # OkCoin has started returning this error on non-executed orders or orders
            # with no fees.
            return Money('0', 'USD')

        # Okcoin returns fees as negative values.
        fee_amount = abs(Decimal(response['data']['fee']))
        fee_currency = 'USD'

        # If the order has not been filled, response['data']['type] is not
        # in the response.
        if 'type' in response['data']:
            fee_currency = response['data']['type'].upper()

        fee = Money(fee_amount, fee_currency)

        return fee

    def open_orders_req(self):
        """
        This implementation assumes that no open orders were placed more than a week
        ago.
        """
        return self.all_transactions_req(page=0, unfilled_orders=True)

    def open_orders_resp(self, req):
        response = self.resp(req)

        raw_open_orders = response['orders']
        open_orders = []

        for raw_order in raw_open_orders:
            mode = self._order_mode_to_const(raw_order['type'])

            usd_price = Money(raw_order['price'], 'USD')
            volume = Money(raw_order['amount'], 'BTC')
            volume_filled = Money(raw_order['deal_amount'], 'BTC')
            remaining = volume - volume_filled

            order = {
                'mode': mode,
                'id': str(raw_order['order_id']),
                'price': usd_price,
                'volume_remaining': remaining
            }

            open_orders.append(order)

        return open_orders

    def order_details(self, order_id):
        reqs = self.order_details_req(order_id)
        return self.order_details_resp(reqs)

    def order_details_req(self, order_id):
        payload = {
            'symbol': 'btc_usd',
            'order_id': order_id,
        }

        reqs = {
            'details': self.req('post', '/order_info.do', data=payload),
            'fee': self.order_fee_req(order_id),
        }

        return reqs

    def order_details_resp(self, reqs):
        details_response = self.resp(reqs['details'])
        fee = self.order_fee_resp(reqs['fee'])

        if details_response['orders']:
            raw_order = details_response['orders'][0]
            order = self.parse_raw_order(raw_order, fee)

            return order
        else:
            empty_order = {
                'time_created': None,
                'type': None,
                'btc_total': Money(0, 'BTC'),
                'fiat_total': Money(0, 'USD'),
                'trades': [],
            }

            return empty_order

    def multi_order_details_req(self, order_ids):
        order_ids_string = ','.join(order_ids)
        details_reqs = []
        fee_reqs = {}

        filled_payload = {
            'symbol': 'btc_usd',
            # we only query for filled orders because this endpoint
            # is used for after-filling accounting
            # okcoin docs say this should be 2 but they're wrong
            'type': self.PARAM_ORDER_STATUS_FILLED,
            'order_id': order_ids_string,
        }

        details_reqs.append(self.req('post', '/orders_info.do', data=filled_payload))

        unfilled_payload = {
            'symbol': 'btc_usd',
            # we only query for filled orders because this endpoint
            # is used for after-filling accounting
            # okcoin docs say this should be 1 but they're wrong
            'type': self.PARAM_ORDER_STATUS_UNFILLED,
            'order_id': order_ids_string,
        }

        details_reqs.append(self.req('post', '/orders_info.do', data=unfilled_payload))

        for order_id in order_ids:
            fee_reqs[order_id] = self.order_fee_req(order_id)

        reqs = {
            'details': details_reqs,
            'fees': fee_reqs,
        }

        return reqs

    def multi_order_details_resp(self, reqs, order_ids_not_used):
        raw_orders = []
        fees = {}

        for req in reqs['details']:
            response = self.resp(req)
            raw_orders += response['orders']

        for order_id, req in reqs['fees'].items():
            response = self.order_fee_resp(req)
            fees[order_id] = response
            
        data = {}

        for raw_order in raw_orders:
            order_id = str(raw_order['order_id'])
            order = self.parse_raw_order(raw_order, fees[order_id])

            data[order_id] = order

        return data

    def parse_raw_order(self, raw_order, fee):
        mode = self._order_mode_to_const(raw_order['type'])
 
        timestamp = int(float(raw_order['create_date'])) / 1000

        btc_total = Money(raw_order['deal_amount'], 'BTC')
        avg_price = Money(raw_order['avg_price'], 'USD')

        total_usd = self.round(avg_price * btc_total.amount)

        # okcoin does not have seperate notions of trade and raw_order
        # so if we see a partially filled raw_order here, treat it as a 
        # full trade

        trades = []

        if total_usd > 0:
            fake_trade = {
                'time': timestamp,
                'trade_id': None,
                'fee': fee,
                'btc': btc_total,
                'fiat': total_usd,
            } 

            trades.append(fake_trade)

        order = {
            'time_created': timestamp,
            'type': mode,
            'btc_total': btc_total,
            'fiat_total': total_usd,
            'trades': trades
        }


        return order

    def cancel_order_req(self, order_id):
        payload = { 
            'symbol': 'btc_usd',
            'order_id': order_id,
        }

        return self.req('post', '/cancel_order.do', data=payload)

    def cancel_order_resp(self, req):
        response = self.resp(req)
        return {'success': True}

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        admin_password = self._load_env('OKCOIN_BTC_USD_ADMIN_PASSWORD')

        payload = {
            'symbol': 'btc_usd',
            'chargefee': '0.001', # generous miner fee, 30 cents
            'trade_pwd': admin_password,
            'withdraw_address': address,
            'withdraw_amount': str(volume.amount),
        }

        return self.req('post', '/withdraw.do', data=payload)

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)
        return {'success': True, 'exchange_withdrawal_id': response['withdraw_id']}

    def process_db_balance_for_audit(self, db_balance):
        # OkCoin truncates to 6 decimal places in their API, so we want do the same
        # to our stored balance before comparing them.
        if db_balance.currency == 'BTC':
            db_balance = db_balance.round_to_decimal_places(6, cdecimal.ROUND_HALF_EVEN)

        return db_balance

    def audit(self, skip_recent=0):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (only include
        orders that have some trades).
        """
        all_orders = self.all_transactions(unfilled_orders=False)
        all_orders = all_orders['orders']

        # list of 100ish previous orders with volume filled > 0
        orders = OrderedDict()

        for order in all_orders:
            order_id = str(order['order_id'])
            status = order['status']
            volume_filled = Money(order['deal_amount'], 'BTC')

            if (status == self.ORDER_STATUS_PARTIALLY_FILLED 
                    or status == self.ORDER_STATUS_FULLY_FILLED):
                orders[order_id] = volume_filled

        return orders




import json
import os

import requests

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

# you probably haven't heard of this exception before
class SlackerException(Exception):
    pass


class Slacker(object):
    def __init__(self, channel, username, icon_url=None, icon_emoji=None, link_names=True):
        self.channel = channel
        self.username = username
        self.icon_url = icon_url
        self.icon_emoji = icon_emoji
        self.link_names = link_names

        self.url = os.environ['SLACK_WEBHOOK_URL']

    def notify(self, message):
        payload = {
            'channel': self.channel,
            'username': self.username,
            'text': message,
        }

        if self.icon_url:
            payload['icon_url'] = self.icon_url

        if self.icon_emoji:
            payload['icon_emoji'] = self.icon_emoji

        if self.link_names is not None:
            payload['link_names'] = self.link_names

        json_payload = json.dumps(payload)

        r = requests.post(self.url, data=json_payload)
        if not r.ok:
            raise SlackerException(r.text)



"""
Unit tests for gryphon.lib.gryphonfury.close_options
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import mock
import sure
import unittest

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_order import Order as ExchangeOrder
from gryphon.lib.gryphonfury import close_options
from gryphon.lib.models.exchange import Exchange
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money


class TestCloseOptions():
    def mockExchange(self):
        exchange = Exchange('TEST')
        exchange.currency = 'USD'
        exchange.market_order_fee = 0.01
        exchange.limit_order_fee = 0.01

        return exchange

    def setUp(self):
        """
        All tests start with a 1 BTC position opened with a single bid.
        """

        self.order = mock.MagicMock()

        self.price = Money('100', 'USD')
        self.fee = Money('0.05', 'USD')
        self.volume = Money('1', 'BTC')

        self.position_trades = [
            Trade(Consts.BID, self.price, self.fee, self.volume, '1', self.order),
        ]

        self.exchange_name = 'BITSTAMP_BTC_USD'

        e = self.mockExchange()

        self.open_orders = {
            'asks': [
                ['101', self.volume, Consts.ASK],
            ],
            'bids': [
                ['99', self.volume, Consts.BID],
            ],
        }

        bids = [
            ExchangeOrder(Money('99', 'USD'), Money('1', 'BTC'), e, Consts.BID),
            ExchangeOrder(Money('98', 'USD'), Money('1', 'BTC'), e, Consts.BID),
        ]

        asks = [
            ExchangeOrder(Money('101', 'USD'), Money('1', 'BTC'), e, Consts.ASK),
            ExchangeOrder(Money('102', 'USD'), Money('1', 'BTC'), e, Consts.ASK),
        ]

        self.orderbook = {'bids': bids, 'asks': asks}

    def tearDown(self):
        pass

    def test_market_order(self):
        revenue, fee = close_options.get_pl_from_market_order(
            self.position_trades,
            self.orderbook,
            self.exchange_name,
        )

        revenue.should.equal(Money('-1', 'USD'))
        fee.should.equal(Money('0.0995', 'USD'))

    def test_limit_order(self):
        revenue, fee = close_options.get_pl_from_limit_order(
            self.position_trades,
            self.orderbook,
            self.exchange_name,
        )

        # These numbers aren't even because the library tests the scenario where we beat
        # the top order by 1 cent.
        revenue.should.equal(Money('0.99', 'USD'))
        fee.should.equal(Money('0.100495', 'USD'))

    def test_open_order(self):
        revenue, fee = close_options.get_pl_from_open_orders(
            self.position_trades,
            self.exchange_name,
            self.open_orders,
        )

        revenue.should.equal(Money('1', 'USD'))
        fee.should.equal(Money('0.1005', 'USD'))



from gryphon.execution.lib.exchange_color import exchange_color
from gryphon.lib.exchange.exchange_factory import *
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib import session

logger = get_logger(__name__)


def balance_requests(exchanges):
    balance_requests = []
    for exchange in exchanges:
        balance_requests.append(exchange.get_balance_req())
    return balance_requests


def balance_responses(exchanges, balance_requests):
    """
    This function uses environment variables to set
    a minimum balances for an exchange. 
    Format:{{exchange.name}}_MINIMUM_USD
    Examples: BITSTAMP_MINIMUM_USD, CAVIRTEX_MINIMUM_BTC
    """
    balances = {}
    balances['system'] = Balance()

    for exchange in exchanges:
        req = balance_requests.pop(0)
        balances[exchange.name] = exchange.get_balance_resp(req)
        balances['system']['USD'] += balances[exchange.name].fiat().to('USD')
        balances['system']['BTC'] += balances[exchange.name]['BTC']
    return balances


def get_db_balances(exchanges):
    db = session.get_a_trading_db_mysql_session()

    db_balances = {}
    db_balances['system'] = Balance()
    try:
        for exchange in exchanges:
            exchange_data = exchange.exchange_account_db_object(db)
            db_balances[exchange.name] = exchange_data.balance
            db_balances['system']['USD'] += db_balances[exchange.name].fiat().to('USD')
            db_balances['system']['BTC'] += db_balances[exchange.name]['BTC']
    finally:
        db.close()
    return db_balances


def format_balances(exchange_balances, db_balances):
    output_string = u"\n{0:15} : {1:15} | {2:15} || {3:15} | {4:15}\n".format("Balances", "FIAT", "BTC", "dbFIAT", "dbBTC")
    for name, balance in sorted(exchange_balances.iteritems()):
        db_balance = db_balances[name]
        chunk = u"{0:15} : {1:15} | {2:15.8f} || {3:15} | {4:15.8f}\n".format(
            name,
            balance.fiat(),
            balance['BTC'].amount,
            db_balance.fiat(),
            db_balance['BTC'].amount
            )
        chunk = exchange_color(chunk, name)
        output_string += chunk
    
    return output_string

    
def balance(exchange_name):
    if exchange_name:
        exchange = make_exchange_from_key(exchange_name)
        exchanges = [exchange]
    else:
        exchanges = all_exchanges()
    brs = balance_requests(exchanges)
    balances = balance_responses(exchanges, brs)

    db_balances = get_db_balances(exchanges)

    print(format_balances(balances, db_balances))



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.bitfinex_trades_poller import BitfinexTrades
from gryphon.data_service.pollers.trades.bitstamp_trades_poller import BitstampTrades
from gryphon.data_service.pollers.trades.coinbase_cad_trades_poller import CoinbaseCadTrades
from gryphon.data_service.pollers.trades.coinbase_trades_poller import CoinbaseTrades
from gryphon.data_service.pollers.trades.gemini_trades_poller import GeminiTrades
from gryphon.data_service.pollers.trades.itbit_trades_poller import ItbitTrades
from gryphon.data_service.pollers.trades.kraken_cad_trades_poller import KrakenCADTrades
from gryphon.data_service.pollers.trades.kraken_trades_poller import KrakenTrades
from gryphon.data_service.pollers.trades.kraken_usd_trades_poller import KrakenUSDTrades
from gryphon.data_service.pollers.trades.okcoin_trades_poller import OkcoinTrades
from gryphon.data_service.pollers.trades.quadriga_trades_poller import QuadrigaTrades
from gryphon.data_service.task import Task


class TradesPollTask(Task):
    def __init__(self, exchanges=[]):
        exchange_pollers = {
            'KRAKEN': KrakenTrades(),
            'KRAKEN_USD': KrakenUSDTrades(),
            'KRAKEN_CAD': KrakenCADTrades(),
            'ITBIT': ItbitTrades(),
            'QUADRIGA': QuadrigaTrades(),
            'OKCOIN': OkcoinTrades(),
            'GEMINI': GeminiTrades(),
            'BITFINEX': BitfinexTrades(),
            'COINBASE_BTC_USD': CoinbaseTrades(),
            'BITSTAMP_BTC_USD': BitstampTrades(),
            'COINBASE_CAD': CoinbaseCadTrades(),
        }

        if exchanges:
            self.pollers = [exchange_pollers[ex] for ex in exchanges]
        else:
            self.pollers = exchange_pollers.values()

    def start_task(self):
        for poller in self.pollers:
            poller.start()



"""
Taken from: https://zapier.com/engineering/profiling-python-boss/
Check out that post for examples of how to use these.

These are two simple decorators that when added to a function, output
a profile of all the function calls made from within that function.
They use two different profiling methods (cprofile vs. line_profiler)
the outputs of each can be different levels of usable for different
cases, so I included both here.
"""
from collections import defaultdict
import functools
import timeit

import monkeypatch_timeit; monkeypatch_timeit.monkeypatch_timeit()
from line_profiler import LineProfiler

def do_profile(follow=[]):
    def inner(func):
        def profiled_func(*args, **kwargs):
            try:
                profiler = LineProfiler()
                profiler.add_function(func)
                for f in follow:
                    profiler.add_function(f)
                profiler.enable_by_count()
                return func(*args, **kwargs)
            finally:
                profiler.print_stats()
        return profiled_func
    return inner


import cProfile

def do_cprofile(func):
    def profiled_func(*args, **kwargs):
        profile = cProfile.Profile()
        try:
            profile.enable()
            result = func(*args, **kwargs)
            profile.disable()
            return result
        finally:
            profile.print_stats(2)
    return profiled_func

# module level storage of function profile times.
# This can then be accessed by anyone who wants to see/print tick_profile results
# maps a function name to a list of profiled times
tick_profile_data = defaultdict(list)

def tick_profile(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        def run_func():
            return func(*args, **kwargs)

        t, result = timeit.timeit(run_func, number=1)
        tick_profile_data[func.__name__].append(t)
        return result
    return inner



from admin_base import AdminBaseHandler
import tornado.web

from mixins.start_and_end_time import StartAndEndTimeMixin
import util.tick_times as tick_times


class BlockTimesHandler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self, exchange_name=None):
        start_time, end_time = self.get_start_time_and_end_time()

        page_title = '%s Tick Block Times' % (
            exchange_name.capitalize(),
        )

        all_block_time_series = tick_times.get_tick_block_time_series_for_exchange(
            self.trading_db,
            exchange_name,
            start_time,
            end_time,
        )

        self.render_template(
            'block_times.html',
            args={
                'page_title': page_title,
                'all_block_time_series': all_block_time_series,
            },
        )



import numpy as np
from exponential_moving_average import ExpMovingAverage


sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData, delimiter=',',unpack=True)

def TR(d,c,h,l,o,yc):
    x = h-l
    y = abs(h-yc)
    z = abs(l-yc)

    print x
    print y
    print z

    if y <= x >= z:
        TR = x
    elif x <= y >= z:
        TR = y
    elif x <= z >= y:
        TR = z

    print d, TR
    return d, TR

x = 1

TRDates = []
TrueRanges = []

while x < len(date):
    TRDate, TrueRange = TR(date[x],closep[x],highp[x],lowp[x],openp[x],closep[x-1])
    TRDates.append(TRDate)
    TrueRanges.append(TrueRange)
    x+=1



print len(TrueRanges)
ATR = ExpMovingAverage(TrueRanges,14)

print ATR



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.orderbook.bitfinex_orderbook_poller import BitfinexOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_bch_btc_orderbook_poller import BitstampBCHBTCOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_bch_btc_orderbook_poller import BitstampBCHBTCOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_bch_eur_orderbook_poller import BitstampBCHEUROrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_bch_usd_orderbook_poller import BitstampBCHUSDOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_btc_eur_orderbook_poller import BitstampBTCEUROrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_eth_btc_orderbook_poller import BitstampETHBTCOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_eth_eur_orderbook_poller import BitstampETHEUROrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_eth_usd_orderbook_poller import BitstampETHUSDOrderbook
from gryphon.data_service.pollers.orderbook.bitstamp_btc_usd_orderbook_poller import BitstampBTCUSDOrderbook
from gryphon.data_service.pollers.orderbook.gemini_eth_btc_orderbook_poller import GeminiETHBTCOrderbook
from gryphon.data_service.pollers.orderbook.gemini_eth_usd_orderbook_poller import GeminiETHUSDOrderbook
from gryphon.data_service.pollers.orderbook.gemini_orderbook_poller import GeminiOrderbook
from gryphon.data_service.pollers.orderbook.itbit_orderbook_poller import ItbitOrderbook
from gryphon.data_service.pollers.orderbook.kraken_cad_orderbook_poller import KrakenCADOrderbook
from gryphon.data_service.pollers.orderbook.kraken_orderbook_poller import KrakenOrderbook
from gryphon.data_service.pollers.orderbook.kraken_usd_orderbook_poller import KrakenUSDOrderbook
from gryphon.data_service.pollers.orderbook.okcoin_orderbook_poller import OkCoinOrderbook
from gryphon.data_service.pollers.orderbook.poloniex_eth_btc_orderbook_poller import PoloniexETHBTCOrderbook
from gryphon.data_service.pollers.orderbook.quadriga_orderbook_poller import QuadrigaOrderbook
from gryphon.data_service.pollers.orderbook.websocket.coinbase_cad_orderbook_websocket import CoinbaseCADOrderbookWebsocket
from gryphon.data_service.pollers.orderbook.coinbase_btc_usd_orderbook_poller import CoinbaseBTCUSDOrderbook
from gryphon.data_service.task import Task


class OrderbookPollTask(Task):
    def __init__(self, exchanges=[]):
        exchange_pollers = {
            'BITFINEX_BTC_USD': BitfinexOrderbook(),
            'BITSTAMP_BCH_EUR': BitstampBCHEUROrderbook(),
            'BITSTAMP_BCH_USD': BitstampBCHUSDOrderbook(),
            'BITSTAMP_BCH_BTC': BitstampBCHBTCOrderbook(),
            'BITSTAMP_BTC_EUR': BitstampBTCEUROrderbook(),
            'BITSTAMP_BTC_USD': BitstampBTCUSDOrderbook(),
            'BITSTAMP_ETH_BTC': BitstampETHBTCOrderbook(),
            'BITSTAMP_ETH_EUR': BitstampETHEUROrderbook(),
            'BITSTAMP_ETH_USD': BitstampETHUSDOrderbook(),
            'COINBASE_BTC_USD': CoinbaseBTCUSDOrderbook(),
            'GEMINI_BTC_USD': GeminiOrderbook(),
            'GEMINI_ETH_BTC': GeminiETHBTCOrderbook(),
            'GEMINI_ETH_USD': GeminiETHUSDOrderbook(),
            'ITBIT_BTC_USD': ItbitOrderbook(),
            'KRAKEN_BTC_EUR': KrakenOrderbook(),
            'KRAKEN_BTC_USD': KrakenUSDOrderbook(),
            'KRAKEN_BTC_CAD': KrakenCADOrderbook(),
            'OKCOIN_BTC_USD': OkCoinOrderbook(),
            'POLONIEX_ETH_BTC': PoloniexETHBTCOrderbook(),
        }

        if exchanges:
            self.pollers = [
                exchange_pollers[ex]
                for ex in exchanges
                if ex in exchange_pollers
            ]
        else:
            self.pollers = exchange_pollers.values()

    def start_task(self):
        for poller in self.pollers:
            poller.start()



import uuid
from gryphon.lib.exchange.base import Exchange
from gryphon.lib.exchange.consts import Consts

class Trade(object):
    __slots__ = ('ask', 'volume', 'trade_id', 'price', 'bid', 'tick')   
    def  __init__(self, price, volume, bid_order, ask_order, trade_id=uuid.uuid4().hex, tick=0):    
         
        self.trade_id = trade_id
        self.bid = bid_order
        self.ask = ask_order
        self.price = price
        self.volume = volume
        self.tick = tick
        
    def __str__(self):
        return (u'[EXCHANGE Trade] - [%s BTC @ %s USD/BTC] - Bid:%s, Ask:%s' % 
            (self.volume, self.price, self.bid.order_id, self.ask.order_id))

    def __repr__(self):
        return "<" + unicode(self) + ">"
    
    @property
    def trade_type(self):
        if self.bid.order_id:
            return Consts.BID
        elif self.ask.order_id:
            return Consts.ASK
        else:
            raise ValueError('We did not participate in this trade so trade_type doesnt mean anything')




# -*- coding: utf-8 -*-
from datetime import datetime
import uuid

from sqlalchemy import Column, Unicode, UnicodeText, DateTime, Integer, Numeric
from sqlalchemy.orm import relationship, backref

from gryphon.lib.models.base import Base
from gryphon.lib.money import Money

metadata = Base.metadata


class Result(Base):
    __tablename__ = 'result'

    result_id = Column(Integer, primary_key=True)
    ticks = Column(Integer, nullable=False)
    unique_id = Column(Unicode(64), nullable=False)
    algorithm = Column(Unicode(64), nullable=False)
    batch = Column(Unicode(64), nullable=False)
    _trading_volume = Column('trading_volume', Numeric(precision=20, scale=10))
    _usd = Column('usd', Numeric(precision=20, scale=10))
    _btc = Column('btc', Numeric(precision=20, scale=10))
    time_created = Column(DateTime, nullable=False)
    
    trades = relationship('ResultTrade', cascade="all,delete-orphan", backref='result')
    
    def __init__(self, usd, btc, trading_volume, algorithm, batch, ticks):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.algorithm = algorithm
        self.batch = batch
        self.usd = usd
        self.btc = btc
        self.ticks = ticks
        self.trading_volume = trading_volume
    
    @property
    def trading_volume(self):
        return Money(self._trading_volume, 'BTC')

    @trading_volume.setter
    def trading_volume(self, value):
        self._trading_volume = value.amount
    
    @property
    def btc(self):
        return Money(self._btc, 'BTC')

    @btc.setter
    def btc(self, value):
        self._btc = value.amount

    @property
    def usd(self):
        return Money(self._usd, 'USD')

    @usd.setter
    def usd(self, value):
        self._usd = value.amount
    



import pyximport; pyximport.install()

import unittest

from cdecimal import Decimal

from gryphon.lib.exchange.kraken_btc_usd import KrakenBTCUSDExchange
from gryphon.lib.money import Money
from gryphon.tests.logic.configuration.exchange import BaseConfiguration


class KrakenUSDConfigurationTest(BaseConfiguration, unittest.TestCase):
    def __init__(self, *args, **kwargs):
        self.exchange_class = KrakenBTCUSDExchange

        self.DEFAULT_MARKET_FEE = Decimal('0.0024')
        self.DEFAULT_FIAT_TOLERANCE = Money('0.0001', 'USD')
        self.DEFAULT_VOLUME_TOLERANCE = Money('0.00000001', 'BTC')

        super(KrakenUSDConfigurationTest, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
import logging
import traceback

import tornado.web

from gryphon.dashboards.handlers.base import BaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.dashboards.models.user import User
from gryphon.dashboards.util.exchanges import load_exchange_names
from gryphon.lib.exchange import exchange_factory
from gryphon.dashboards.handlers.strategies.builtin_configs import BUILTIN_STRAT_CONFIGS

logger = logging.getLogger(__name__)


class AdminBaseHandler(BaseHandler):
    """
    A class to collect common handler methods - all other handlers should subclass this
    one.
    """
    def write_error(self, status_code, **kwargs):
        """
        Override to implement custom error pages.
        """
        if 'exc_info' in kwargs and status_code not in [403, 404, 405]:
            exc_info = kwargs.get('exc_info')

            error_readout = [
                unicode(line, 'utf8') for line in traceback.format_exception(*exc_info)
            ]

            logger.critical(u''.join(error_readout))

        self.render_template('error.html', args={})

    def get_current_user(self):
        if not hasattr(self, 'dashboard_db'):
            return None

        if not hasattr(self, '_current_user'):
            self._current_user = self.dashboard_db.query(User)\
                .filter_by(unique_id=self.get_secure_cookie('user'))\
                .first()

        return self._current_user

    def get_secure_cookie(self, name, include_name=True, value=None):
        cookie_value = super(AdminBaseHandler, self).get_secure_cookie(name)
        return unicode(cookie_value or '', 'utf8')

    def show_error_message(self, message):
        self.set_secure_cookie('error_message', message)

    def show_message(self, message):
        self.set_secure_cookie('message', message)

    def login_user(self, user):
        self._current_user = user
        self.set_secure_cookie('user', user.unique_id)

    def logout_user(self):
        self._current_user = None
        self.clear_cookie('user')

    def render_template(self, template, **kwargs):
        # Grab the cookie messages.
        current_user = self.get_current_user()
        cookie_message = self.get_secure_cookie('message')
        error_message = self.get_secure_cookie('error_message')
        self.set_secure_cookie('message', u'')
        self.set_secure_cookie('error_message', u'')

        kwargs.update({
            'user': current_user,
            'error_message': error_message,
            'message': cookie_message,
            'xsrf_token': self.xsrf_token,
        })

        exchange_names_map = load_exchange_names(self.trading_db)
        datum_links = self.datum_links(exchange_names_map)

        if 'args' not in kwargs:
            kwargs['args'] = {}

        custom_strategies = {
            key: val for key, val in self.configuration['strategies'].items()
            if key not in BUILTIN_STRAT_CONFIGS
        }

        kwargs['args'].update({
            'exchanges': exchange_names_map,
            'bank_account_keys': exchange_factory.BANK_ACCOUNT_KEYS,
            'datum_links': datum_links,
            'custom_strategies': custom_strategies,
        })

        if isinstance(self, StartAndEndTimeMixin):
            start_time, end_time = self.get_start_time_and_end_time()

            kwargs['args']['start_end_widget'] = {
                'start_time': start_time,
                'end_time': end_time,
            }

        template = 'templates/%s' % template

        return self.render(template, **kwargs)

    def datum_links(self, exchange_names_map):
        exchange_keys = [k.upper() for k in exchange_names_map]

        spread_datum_keys = []
        ob_delay_datum_keys = []

        for exchange_key in exchange_keys:
            spread_datum_keys.append('%s_NATIVE_FUNDAMENTAL_VALUE' % exchange_key)
            ob_delay_datum_keys.append('%s_ORDERBOOK_DELAY' % exchange_key)

        spread_url = '/gryphon-fury/datum/%s' % '~'.join(spread_datum_keys)
        ob_delay_url = '/gryphon-fury/datum/%s' % '~'.join(ob_delay_datum_keys)

        links = [
            ['Spreads', spread_url],
            ['Orderbook Delays', ob_delay_url],
        ]

        return links


class Handler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self):
        start_time, end_time = self.get_start_time_and_end_time()



# -*- coding: utf-8 -*-
import datetime
import pytz
import os

import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.logger import get_logger
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction

logger = get_logger(__name__)

interval = datetime.timedelta(seconds=86400)
pwd = os.path.dirname(os.path.realpath(__file__))


class LedgerHandler(AdminBaseHandler, StartAndEndTimeMixin):
    def __init__(self, *args, **kwargs):
        super(LedgerHandler, self).__init__(*args, **kwargs)

        # This dictionary maps bitcoin addresses to exchange names.
        # TODO: Make this user-configurable.
        self.address_map = {}

    @tornado.web.authenticated
    def get(self, exchange_name=None, currency=None):
        exchange_name = exchange_name.upper()
        start_time, end_time = self.get_start_time_and_end_time()

        if (exchange_name
                not in exchange_factory.initialized_ledgers(self.trading_db)):
            self.redirect('/404')
            return

        fiat_currency, currency, ledger_table = self.get_ledger_table_for_time(
            exchange_name,
            start_time,
            end_time,
            currency,
        )

        self.render_template(
            'ledger.html',
            args={
                'exchange_name': exchange_name,
                'fiat_currency': fiat_currency,
                'currency': currency,
                'ledger_table': ledger_table,
            },
        )

    def get_ledger_table_for_time(self, exchange_name, start_time, end_time, currency=None):
        if currency:
            currency = currency.upper()

        exchange_name = exchange_name.capitalize()
        exchange = exchange_factory.make_exchange_from_key(exchange_name)
        exchange_data = exchange_factory.make_exchange_data_from_key(
            exchange_name,
            self.trading_db,
        )

        if exchange:
            # Normal bitcoin exchange.
            fiat_currency = exchange.currency
        else:
            # Bank Accounts don't have exchange objects.
            fiat_currency = self.currency_for_bank_account(exchange_name)

            # Bank Accounts only have one currency, so we set a currency filter by
            # default. It just makes the UI look nicer instead of having an empty BTC
            # column.
            currency = fiat_currency

        ledger = exchange_data.ledger(
            start_time=start_time,
            end_time=end_time,
            currency=currency,
        )

        # ledger is empty for this time period.
        if len(ledger) == 0:
            return fiat_currency, currency, []

        # get the exchange balance from before these ledger entries started
        oldest_entry = ledger[-1]
        oldest_time = oldest_entry.time_created

        # There can be multiple entries at the same time
        entries_at_oldest_time = []

        for entry in ledger:
            if isinstance(entry, Trade) and entry.time_created == oldest_time:
                entries_at_oldest_time.append(entry)
            elif isinstance(entry, Transaction) and entry.time_completed == oldest_time:
                entries_at_oldest_time.append(entry)

        oldest_time = pytz.utc.localize(oldest_time)
        balance_at_oldest_time = exchange_data.ledger_balance(end_time=oldest_time)

        # balance_at_oldest_time includes all the entries_at_oldest_time
        # so we need to remove them to get the pre-ledger balance
        pre_ledger_balance = balance_at_oldest_time

        for entry in entries_at_oldest_time:
            pre_ledger_balance -= entry.position

        starting_ledger_balance = exchange_data.ledger_balance(end_time=start_time)

        ledger_table = []

        for entry in ledger:
            if isinstance(entry, Trade):
                table_entries = self.table_entries_from_trade(entry)
            elif isinstance(entry, Transaction):
                table_entries = self.table_entries_from_transaction(entry)

            ledger_table += table_entries

        ledger_table = self.filter_to_currency(ledger_table, currency)

        current_balance = starting_ledger_balance

        for table_entry in reversed(ledger_table):
            if 'credit' in table_entry:
                current_balance += table_entry['credit']
            if 'debit' in table_entry:
                current_balance -= table_entry['debit']

            table_entry['balance'] = current_balance

        return fiat_currency, currency, ledger_table

    def get_exchange_for_address(self, address):
        """
        TODO: allow users to configure a list of exchange deposit addresses at startup
          so we can tell the intended destination of withdrawals.
        """

        if address and address in self.address_map:
            return address_map[address]
        else:
            return 'External transfer'

    def currency_for_bank_account(self, bank_account_name):
        """
        Get the operating currency of our bank accounts.

        Hacky, but I don't want to add exchange objects for all of these
        """
        if bank_account_name.upper() in ['BMO_CAD', 'BMO_CAD_OPS']:
            return 'CAD'
        else:
            return 'USD'

    def filter_to_currency(self, ledger_table, currency):
        if not currency:
            return ledger_table

        filtered_table = []
        for entry in ledger_table:
            if ('credit' in entry and entry['credit'].currency == currency
                    or 'debit' in entry and entry['debit'].currency == currency):
                filtered_table.append(entry)

        return filtered_table

    def table_entries_from_trade(self, trade):
        entries = []

        if trade.trade_type == Trade.BID:
            desc = 'Trade #%s: Buy %s for %s' % (
                trade.exchange_trade_id,
                trade.volume,
                trade.price,
            )

            entries.append({
                'credit': trade.volume,
                'description': desc,
                'type': 'Trade - BUY'
            })
            entries.append({
                'debit': trade.price,
                'description': desc,
                'type': 'Trade - BUY',
            })

        elif trade.trade_type == Trade.ASK:
            desc = 'Trade #%s: Sell %s for %s' % (
                trade.exchange_trade_id,
                trade.volume,
                trade.price,
            )
            entries.append({
                'credit': trade.price,
                'description': desc,
                'type': 'Trade - SELL'
            })
            entries.append({
                'debit': trade.volume,
                'description': desc,
                'type': 'Trade - SELL',
            })

        if trade.fee:
            entries.append({
                'debit': trade.fee,
                'description': 'Trade #%s: Fee' % trade.exchange_trade_id,
                'type': 'Trade - FEE',
            })

        date = trade.time_created
        for entry in entries:
            entry['date'] = date
            entry['details'] = ''

        return entries

    def table_entries_from_transaction(self, transaction):
        entries = []

        if 'drift' in transaction.transaction_details:
            # Skipping Drift Transactions
            return entries

        if transaction.transaction_type == Transaction.WITHDRAWL:
            deposit_address = transaction.transaction_details.get('deposit_address')
            destination = self.get_exchange_for_address(deposit_address)
            desc = 'Withdrawal to %s' % destination

            entries.append({
                'debit': transaction.amount,
                'description': desc,
                'type': 'Withdrawal - %s' % destination,
            })

        elif transaction.transaction_type == Transaction.DEPOSIT:
            desc = 'Deposit'
            entries.append({
                'credit': transaction.amount,
                'description': desc,
                'type': 'Deposit',
            })

        if transaction.fee:
            entries.append({
                'debit': transaction.fee,
                'description': 'Transaction Fee',
                'type': 'Transaction Fee',
            })

        date = transaction.time_completed
        if not date:
            date = transaction.time_created
        for entry in entries:
            entry['date'] = date
            entry['details'] = ''.join([
                '%s:%s ' % (k, v)
                for k, v in transaction.transaction_details.iteritems()
                if k in ['external_transaction_id', 'notes'] and v not in ['xxx']
            ])

        return entries



# -*- coding: utf-8 -*-
import time
import os
import datetime
import requests
import hmac
import urllib
import hashlib
import base64
import json
import math
from urllib import urlencode
from collections import OrderedDict
from delorean import Delorean, parse

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money
from base import *
from exceptions import *
from exchange_order import Order
from gryphon.lib.models.exchange import Balance

from cdecimal import *
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class BitmeExchange(Exchange):
    def __init__(self, session=None):
        super(BitmeExchange, self).__init__(session)
        self.name = u'BITME'
        self.friendly_name = u'bitme'
        self.base_url = 'https://bitme.com/rest'
        self.currency = "USD"
        self.fee = Decimal("0")
        self.bid_string = "BID"
        self.ask_string = "ASK"



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse


class ItbitTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'ITBIT_BTC_USD'
        self.url = 'https://api.itbit.com/v1/markets/XBTUSD/trades?since=0'
        self.poll_time = 10

    def parse_trades(self, resp_obj):
        raw_trades = resp_obj['recentTrades']
        trades = []

        for raw_trade in raw_trades:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(raw_trade['price'], 'USD'),
                'volume': Money(raw_trade['amount'], 'BTC'),
                'timestamp': parse(raw_trade['timestamp']).epoch,
                'trade_id': raw_trade['matchNumber'],
            }

            trades.append(trade)

        return trades



import termcolor as tc


EXCHANGE_COLORS = {
    "BITSTAMP": lambda t: tc.colored(t, "green"),
    "COINBASE": lambda t: tc.colored(t, "blue"),
    "KRAKEN": lambda t: tc.colored(t, "cyan"),
    "CAVIRTEX": lambda t: tc.colored(t, "red"),
    "QUADRIGA": lambda t: tc.colored(t, "yellow"),
    "BITFINEX": lambda t: tc.colored(t, attrs=['bold']),
    "ITBIT": lambda t: tc.colored(t, "magenta"),
    "OKCOIN": lambda t: tc.colored(t, "red", attrs=['bold']),
}


def exchange_color(text, exchange_name):
    try:
        colored_text = EXCHANGE_COLORS[exchange_name](text)
    except:
        colored_text = text
    return colored_text


def legend():
    output = ""
    for exchange_name in EXCHANGE_COLORS:
        output += exchange_color(exchange_name, exchange_name) + "  "
    return output



"""
Just a few exercises for the execution configuration helper library.
"""

import pyximport; pyximport.install()
import os
import time
import unittest
import sure

from cdecimal import Decimal

from gryphon.execution.lib import config_helper
from gryphon.lib.money import Money


class TestConfiguration(unittest.TestCase):
    def test_cl_none_doesnt_override_conf_file(self):
        conf_file = {
            'platform': {'audit': True},
            'strategy': {'tick_sleep': 1},
        }

        command_line = {
            'platform': {'audit': None},
            'strategy': {'tick_sleep': None},
        }

        output = config_helper.combine_file_and_command_line_config(
            conf_file,
            command_line,
        )

        assert conf_file['platform']['audit'] is True
        assert conf_file['strategy']['tick_sleep'] == 1

    def test_command_line_overrides_conf_file(self):
        conf_file = {
            'platform': {'audit': False},
            'strategy': {'tick_sleep': 1},
        }

        command_line = {
            'platform': {'audit': True},
            'strategy': {'tick_sleep': 2},
        }

        output = config_helper.combine_file_and_command_line_config(
            conf_file,
            command_line,
        )

        assert conf_file['platform']['audit'] is True
        assert conf_file['strategy']['tick_sleep'] == 2

    def test_combine_works(self):
        conf_file = {
            'platform': {'audit': True},
            'strategy': {'spread': Decimal('0.10')},
        }

        command_line = {
            'platform': {'sentry': True},
            'strategy': {'tick_sleep': None},
        }

        output = config_helper.combine_file_and_command_line_config(
            conf_file,
            command_line,
        )

        assert conf_file['platform']['audit'] is True
        assert conf_file['platform']['sentry'] is True
        assert conf_file['strategy']['spread'] == Decimal('0.10')
        assert conf_file['strategy']['tick_sleep'] is None


    def test_parse_extra_args(self):
        extra_args = ['--spread', '0.01']

        parsed = config_helper.parse_extra_strategy_args(extra_args)

        assert len(parsed.keys()) == 1
        assert parsed.keys()[0] == 'spread'
        assert parsed['spread'] == Decimal('0.01')

    def test_parse_extra_args_boolean(self):
        extra_args = ['--market_order']

        parsed = config_helper.parse_extra_strategy_args(extra_args)

        assert len(parsed.keys()) == 1
        assert parsed.keys()[0] == 'market_order'
        assert parsed['market_order'] == True

    def test_parse_extra_args_complex(self):
        extra_args = [
            '--spread', '0.1',
            '--market_order',
            '--exchange', 'bitstamp',
            '--volume', 'BTC 1.0',
        ]

        parsed = config_helper.parse_extra_strategy_args(extra_args)

        assert len(parsed.keys()) == 4
        assert parsed['spread'] == Decimal('0.1')
        assert parsed['market_order'] == True
        assert parsed['exchange'] == 'bitstamp'
        assert parsed['volume'] == Money('1.0', 'BTC')

    def test_standardization(self):
        base_config = {}

        new_config = config_helper.format_file_config_to_standard(base_config)

        new_config.should.equal({'platform': {}, 'strategy': {}, 'exchanges': {}})

    def test_standardization_trivial(self):
        base_config = {
            'platform': {'audit': False},
        }

        new_config = config_helper.format_file_config_to_standard(base_config)

        new_config.should.equal(
            {'platform': {'audit': False}, 'strategy': {}, 'exchanges': {}}
        )

    def test_standardization_simple(self):
        base_config = {
            'platform': {'audit': False},
            'strategy': {'tick_sleep': Decimal('1')},
        }

        new_config = config_helper.format_file_config_to_standard(base_config)

        set(new_config.keys()).should.equal(set(['platform', 'strategy', 'exchanges']))
        new_config['platform']['audit'].should.equal(False)
        new_config['strategy']['tick_sleep'].should.equal(Decimal('1'))
        new_config['exchanges'].should.equal({})

    def test_standardization_exchanges(self):
        base_config = {
            'platform': {'audit': False},
            'strategy': {'tick_sleep': Decimal('1')},
            'coinbase_btc_usd': {'fiat_balance_tolerance': Money('0.01', 'USD')}
        }

        new_config = config_helper.format_file_config_to_standard(base_config)

        len(new_config['exchanges']).should.equal(1)
        len(new_config['exchanges']['coinbase_btc_usd'].keys()).should.equal(1)
        new_config['exchanges']['coinbase_btc_usd']['fiat_balance_tolerance']\
            .should.equal(Money('0.01', 'USD'))



import pyximport; pyximport.install()

import os
import unittest
import sure

from cdecimal import Decimal

from gryphon.lib import market_making
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.money import Money
from gryphon.lib.exchange.coinbase_btc_usd import CoinbaseBTCUSDExchange


class TestMarketMaking(unittest.TestCase):
    def setUp(self):
        self.empty_book = {
            'bids': [],
            'asks': [],
        }

        self.exchange = CoinbaseBTCUSDExchange()

    def basic_book(self,price_currency='USD', vol_currency='BTC'):
        def price(val):
            return Money(val, price_currency)

        def vol(val):
            return Money(val, vol_currency)

        bids = [Order(price('249'), vol('10'), self.exchange, Order.BID)]
        asks = [Order(price('251'), vol('10'), self.exchange, Order.ASK)]

        return {'bids': bids, 'asks': asks}

    def bigger_book(self, price_currency='USD', vol_currency='BTC'):
        def price(value):
            return Money(value, price_currency)

        def vol(value):
            return Money(value, vol_currency)

        bids = [
            Order(price('249'), vol('10'), self.exchange, Order.BID),
            Order(price('248'), vol('10'), self.exchange, Order.BID)
        ]

        asks = [
            Order(price('251'), vol('10'), self.exchange, Order.ASK),
            Order(price('260'), vol('10'), self.exchange, Order.ASK),
        ]

        return {'bids': bids, 'asks': asks}

    def test_fixed_spread_basic(self):
        result = market_making.midpoint_centered_fixed_spread(
            self.basic_book(),
            Decimal('0.01'),
        )

        result[0].should.equal(Money('247.5', 'USD'))
        result[1].should.equal(Money('252.5', 'USD'))

    def test_fixed_spread_thin(self):
        result = market_making.midpoint_centered_fixed_spread(
            self.basic_book(),
            Decimal('0.0001'),
        )

        result[0].should.equal(Money('249.975', 'USD'))
        result[1].should.equal(Money('250.025', 'USD'))

    def test_fixed_spread_wide(self):
        result = market_making.midpoint_centered_fixed_spread(
            self.basic_book(),
            Decimal('0.5'),
        )

        result[0].should.equal(Money('125', 'USD'))
        result[1].should.equal(Money('375', 'USD'))

    def test_fixed_spread_crypto_price(self):
        result = market_making.midpoint_centered_fixed_spread(
            self.basic_book(price_currency='BTC', vol_currency='ETH'),
            Decimal('0.0001'),
        )

        result[0].should.equal(Money('249.975', 'BTC'))
        result[1].should.equal(Money('250.025', 'BTC'))

    def test_fixed_spread_crypto_price_quote_depth(self):
        result = market_making.midpoint_centered_fixed_spread(
            self.bigger_book(price_currency='BTC', vol_currency='ETH'),
            Decimal('0.0001'),
            quote_depth=Money('20', 'ETH'),
        )

        result[0].should.equal(Money('251.9748', 'BTC'))
        result[1].should.equal(Money('252.0252', 'BTC'))

    def test_no_position(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'BTC'),
            Money('0', 'BTC'),
        )

        result[0].should.equal(Money('2', 'BTC'))
        result[1].should.equal(Money('2', 'BTC'))

    def test_long_position(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'BTC'),
            Money('1', 'BTC'),
        )

        result[0].should.equal(Money('1', 'BTC'))
        result[1].should.equal(Money('2', 'BTC'))

    def test_short_position(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'BTC'),
            Money('-1', 'BTC'),
        )

        result[0].should.equal(Money('2', 'BTC'))
        result[1].should.equal(Money('1', 'BTC'))

    def test_over_short_position(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'BTC'),
            Money('-3', 'BTC'),
        )

        result[0].should.equal(Money('2', 'BTC'))
        result[1].should.equal(Money('0', 'BTC'))

    def test_over_long_position(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'BTC'),
            Money('3', 'BTC'),
        )

        result[0].should.equal(Money('0', 'BTC'))
        result[1].should.equal(Money('2', 'BTC'))

    def test_over_short_position_non_btc(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'ETH'),
            Money('-3', 'ETH'),
        )

        result[0].should.equal(Money('2', 'ETH'))
        result[1].should.equal(Money('0', 'ETH'))

    def test_over_short_position_fiat(self):
        result = market_making.simple_position_responsive_sizing(
            Money('2', 'USD'),
            Money('-3', 'USD'),
        )

        result[0].should.equal(Money('2', 'USD'))
        result[1].should.equal(Money('0', 'USD'))




from collections import namedtuple

from cdecimal import Decimal
from delorean import Delorean
from sqlalchemy import func, or_, and_

from gryphon.lib.exchange import exchange_factory
from gryphon.lib.gryphonfury import positions
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.exchange import Position
from gryphon.lib.models.liability import Liability
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)

# If your ledger is only accurate after a certain time, put it here, and ledger_balance
# requests earlier than this will throw an exception.
EARLIEST_CORRECT_LEDGER_DATE = parse('2015-01-01').datetime


# This number is used by the overwatch bot to determine if there is a bitcoin amount
# missing from our system.
# TODO:
#   - This should be configurable by the user. Since this is only used in overwatch,
#     maybe it should be part of an overwatch config.
#   - We should also be able to represent targets in all our supported cryptocurrencies
#     in this manner.
BTC_NET_ASSETS_TARGET = Money('0', 'BTC')

NO_BURN_ACCOUNT_WARNING = """\
There is no entry with the name 'BURN' in your exchange account table, so we won't\
be able to show non-trading expenses in the dashboard service. You can learn how to\
set this feature up in the framework documentation.\
"""


def get_all_liabilities(trading_db):
    liabilities = trading_db.query(Liability).all()
    return liabilities


def get_active_liabilities(trading_db, date=Delorean().naive):
    debts = get_all_liabilities(trading_db)
    active_debts = []

    # Annoying that this is still necessary.
    date = date.replace(tzinfo=None)

    for debt in debts:
        # an active debt started before the current date
        # and either has not been repaid, or was repaid after the current date

        if (debt.time_started <= date
                and (not debt.time_repayed or debt.time_repayed > date)):
            active_debts.append(debt)

    return active_debts


def get_active_btc_liabilities(trading_db):
    active_liabilities = get_active_liabilities(trading_db)
    active_btc_liabilities = [
        l for l in active_liabilities if l.amount.currency == 'BTC'
    ]
    return active_btc_liabilities


def calculate_btc_net_assets_error(db):
    btc_net_assets = calculate_btc_net_assets(db)
    outstanding_btc_fees = calculate_outstanding_btc_fees(db)

    # outstanding btc fees will be bought back with a buyback transaction
    # so they are like future assets
    error = BTC_NET_ASSETS_TARGET - (btc_net_assets + outstanding_btc_fees)
    return error


def calculate_outstanding_btc_fees(db):
    raw_trade_btc_fees = db.query(func.sum(Trade._fee))\
        .filter(Trade.has_outstanding_btc_fee)\
        .scalar()

    raw_transaction_btc_fees = db.query(func.sum(Transaction._fee))\
        .filter(Transaction.has_outstanding_btc_fee)\
        .scalar()

    if raw_trade_btc_fees == None:
        raw_trade_btc_fees = 0
    if raw_transaction_btc_fees == None:
        raw_transaction_btc_fees = 0

    trade_btc_fees = Money(raw_trade_btc_fees, 'BTC')
    transaction_btc_fees = Money(raw_transaction_btc_fees, 'BTC')

    return trade_btc_fees + transaction_btc_fees


def calculate_btc_net_assets(db):
    exchange_datas = exchange_factory.all_exchange_datas(db)
    assets_from_balance = sum([e.balance['BTC'] for e in exchange_datas])

    # TODO: move this into some sort of common query
    pending_deposits = db.query(Transaction)\
        .filter_by(transaction_status='IN_TRANSIT')\
        .join(ExchangeData)\
        .filter(or_(
            Transaction._amount_currency == 'BTC',
            Transaction._fee_currency == 'BTC',
        ))\
        .all()

    assets_from_pending_deposits = Money(0, 'BTC')
    for pending_deposit in pending_deposits:
        # It is theoretically possible to be charged BTC fees on a fiat transaction
        # or vice versa, so we need to handle them independently
        if pending_deposit.amount.currency == 'BTC':
            assets_from_pending_deposits += pending_deposit.amount

        if pending_deposit.fee and pending_deposit.fee.currency == 'BTC':
            assets_from_pending_deposits -= pending_deposit.fee

    # We discount position by adding it as a liability
    # This cancels out its affect on our balance so our net assets aren't always
    # bouncing around.
    position = positions.cached_multi_position(db)
    liabilities_from_position = position

    bitcoin_liabilities = get_active_btc_liabilities(db)
    liabilities_from_debts = sum([l.amount for l in bitcoin_liabilities])

    logger.debug('Assets from Balance: %s' % assets_from_balance)
    logger.debug('Assets from Pending Deposits: %s' % assets_from_pending_deposits)

    logger.debug('Liabilities from Position: %s' % liabilities_from_position)
    logger.debug('Liabilities from Loans: %s' % liabilities_from_debts)

    total_assets = assets_from_balance + assets_from_pending_deposits
    total_liabilities = liabilities_from_position + liabilities_from_debts

    net_assets = total_assets - total_liabilities

    return net_assets


def ledger_balance(db, start_time=None, end_time=None, include_pending=False, exchange_names=[]):
    """
    Returns the exchange's balance as calculated from its ledger of trades and
    transactions.

    With no params specified we return the current balance.
    With end_time specified we return the balance at a historical point.
    With start_time specified we return a balance diff which can be useful for
    performance (if you know the balance on one day, you can get the next day's balance
    by only looking at that next days ledgers instead of from the beginning).
    """

    if start_time and start_time < EARLIEST_CORRECT_LEDGER_DATE:
        raise ValueError(
            'start_time must be later than %s' % EARLIEST_CORRECT_LEDGER_DATE,
        )

    if end_time and end_time < EARLIEST_CORRECT_LEDGER_DATE:
        raise ValueError(
            'end_time must be later than %s' % EARLIEST_CORRECT_LEDGER_DATE,
        )

    exchange_ids = []

    for exchange_name in exchange_names:
        exchange_data = exchange_factory.make_exchange_data_from_key(exchange_name, db)
        exchange_id = exchange_data.exchange_id
        exchange_ids.append(exchange_id)

    # Trades
    bid_prices = trade_position_query(
        db,
        Trade._price,
        Trade._price_currency,
        Trade.BID,
        start_time=start_time,
        end_time=end_time,
        exchange_names=exchange_names,
    )

    bid_volumes = trade_position_query(
        db,
        Trade._volume,
        Trade._volume_currency,
        Trade.BID,
        start_time=start_time,
        end_time=end_time,
        exchange_names=exchange_names,
    )

    ask_prices = trade_position_query(
        db,
        Trade._price,
        Trade._price_currency,
        Trade.ASK,
        start_time=start_time,
        end_time=end_time,
        exchange_names=exchange_names,
    )

    ask_volumes = trade_position_query(
        db,
        Trade._volume,
        Trade._volume_currency,
        Trade.ASK,
        start_time=start_time,
        end_time=end_time,
        exchange_names=exchange_names,
    )

    trade_fees = trade_position_query(
        db,
        Trade._fee,
        Trade._fee_currency,
        start_time=start_time,
        end_time=end_time,
        exchange_names=exchange_names,
    )

    # Transactions
    deposits = transaction_position_query(
        db,
        Transaction._amount,
        Transaction._amount_currency,
        transaction_type=Transaction.DEPOSIT,
        start_time=start_time,
        end_time=end_time,
        include_pending=include_pending,
        exchange_ids=exchange_ids,
    )

    withdrawals = transaction_position_query(
        db,
        Transaction._amount,
        Transaction._amount_currency,
        transaction_type=Transaction.WITHDRAWL,
        start_time=start_time,
        end_time=end_time,
        include_pending=include_pending,
        exchange_ids=exchange_ids,
    )

    transaction_fees = transaction_position_query(
        db,
        Transaction._fee,
        Transaction._fee_currency,
        start_time=start_time,
        end_time=end_time,
        include_pending=include_pending,
        exchange_ids=exchange_ids,
    )

    logger.debug('Bid Prices: %s' % bid_prices)
    logger.debug('Bid Volumes: %s' % bid_volumes)
    logger.debug('Ask Prices: %s' % ask_prices)
    logger.debug('Ask Volumes: %s' % ask_volumes)
    logger.debug('Trade Fees: %s' % trade_fees)

    logger.debug('Deposits: %s' % deposits)
    logger.debug('Withdrawals: %s' % withdrawals)
    logger.debug('Transaction Fees: %s' % transaction_fees)

    positive_position = sum([
        bid_volumes,
        ask_prices,
        deposits,
    ], Position())

    negative_position = sum([
        bid_prices,
        ask_volumes,
        trade_fees,
        withdrawals,
        transaction_fees,
    ], Position())

    total_position = positive_position - negative_position
    return total_position


def trade_position_query(db, amount_field, currency_field, trade_type=None, start_time=None, end_time=None, exchange_names=[]):
    """
    This sums up a trade Money field which can have multiple currencies.

    Takes an optional trade_type to filter on bids or asks

    Returns a multi-currency Position object
    """
    # Money fields in our database can be in multiple currencies
    # For example, trade fees can be in BTC or USD, so doing a simple sum doesn't work.
    # We group by the currency field, giving us a result like:
    # [('BTC', 10) ('USD', 900)]
    # We convert this to a position object and return it.
    query = db.query(currency_field, func.sum(amount_field))\
        .join(Order)\
        .group_by(currency_field)

    if trade_type:
        query = query.filter(Trade.trade_type == trade_type)

    if start_time:
        query = query.filter(Trade.time_created > start_time)

    if end_time:
        query = query.filter(Trade.time_created <= end_time)

    if exchange_names:
        # maybe join here?
        query = query.filter(Order._exchange_name.in_(exchange_names))

    result = query.all()

    return query_result_to_position(result)


def transaction_position_query(db, amount_field, currency_field, transaction_type=None, start_time=None, end_time=None, include_pending=False, exchange_ids=[]):
    """
    This sums up a transaction Money field which can have multiple currencies.

    Takes an optional transaction_type to filter on deposits or withdrawals

    Returns a multi-currency Position object
    """
    # See comment on trade_position_query above
    query = db.query(currency_field, func.sum(amount_field))

    if include_pending:
        query = query.filter(or_(
            Transaction.transaction_status == Transaction.COMPLETED,
            Transaction.transaction_status == Transaction.IN_TRANSIT,
        ))
    else:
        query = query.filter(Transaction.transaction_status == Transaction.COMPLETED)

    if transaction_type:
        query = query.filter(Transaction.transaction_type == transaction_type)

    if start_time:
        # when include_pending is True we treat all transactions as being completed
        # on their creation time (i.e. instantly)
        if include_pending:
            query = query.filter(Transaction.time_created > start_time)
        else:
            query = query.filter(or_(
                and_(
                    Transaction.time_completed != None,
                    Transaction.time_completed > start_time,
                ),
                and_(
                    Transaction.time_completed == None,
                    Transaction.time_created > start_time,
                ),
            ))

    if end_time:
        if include_pending:
            query = query.filter(Transaction.time_created <= end_time)
        else:
            # We include transactions in the ledger based on when they were completed,
            # not created. This is an issue because we only started recording
            # time_completed on 2015-10-08. So for these pre-time_completed
            # transactions we use their time_created instead.
            query = query.filter(or_(
                and_(
                    Transaction.time_completed != None,
                    Transaction.time_completed <= end_time,
                ),
                and_(
                    Transaction.time_completed == None,
                    Transaction.time_created <= end_time,
                ),
            ))

    if exchange_ids:
        query = query.filter(Transaction.exchange_id.in_(exchange_ids))

    query = query.group_by(currency_field)

    result = query.all()

    return query_result_to_position(result)


def query_result_to_position(db_query_result):
    """
    Convert a query result into a multi-currency position
    """
    position = Position()
    # db_query_result looks like
    # [('BTC', Decimal('10')),
    # ('USD', Decimal('900'))]
    for row in db_query_result:
        currency = row[0]
        amount = row[1]
        if amount:
            position[currency] = Money(amount, currency)
    return position

def get_burn_transactions(db, start_time, end_time):
    """
    Get the list of transactions associated with the 'BURN' account in the databse. If
    there is no burn account, this function will return None.
    """
    burn_exchange_data = None

    try:
        burn_exchange_data = exchange_factory.make_exchange_data_from_key('BURN', db)
    except Exception as e:
        logger.info(NO_BURN_ACCOUNT_WARNING)

        return None

    burn_exchange_id = burn_exchange_data.exchange_id

    burn_transactions = db.query(Transaction)\
        .filter(Transaction.transaction_status == Transaction.COMPLETED)\
        .filter(Transaction.time_completed >= start_time)\
        .filter(Transaction.time_completed < end_time)\
        .filter(Transaction.exchange_id == burn_exchange_id)\
        .order_by(Transaction.time_completed.desc())\
        .all()

    return burn_transactions

def calculate_burn(db, start_time, end_time):
    burn = ledger_balance(
        db,
        start_time=start_time,
        end_time=end_time,
        exchange_names=['BURN'],
    )

    # TODO: make this more accurate.
    midpoint_time = start_time + (start_time - end_time) / 2
    usd_burn = burn.total_usd_value(date=midpoint_time)

    return usd_burn


def calculate_forex_flux(starting_balance, final_balance, start_time, end_time):
    average_balance = starting_balance + final_balance
    for currency in average_balance.keys():
        average_balance[currency] /= 2

    usd_value_at_start = average_balance.total_usd_value(date=start_time)
    usd_value_at_end = average_balance.total_usd_value(date=end_time)

    flux = usd_value_at_end - usd_value_at_start

    return flux




import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import os
import unittest
import sure
import mock
from decimal import Decimal

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.trade import Trade

import logging
logger = logging.getLogger(__name__)

class TestTrade(unittest.TestCase):
    def setUp(self):
        self.order = mock.MagicMock()

        self.price = Money(100, "USD")
        self.fee = Money(1, "USD")
        self.volume = Money(1, "BTC")
        self.bid_trade = Trade(Consts.BID, self.price, self.fee, self.volume, "1", self.order)
        self.ask_trade = Trade(Consts.ASK, self.price, self.fee, self.volume, "2", self.order)

    def tearDown(self):
        pass

    def test_bid_position(self):
        self.bid_trade.position.should.have.key('USD')
        self.bid_trade.position.should.have.key('BTC')
        self.bid_trade.position['USD'].should.equal(Money(-101, "USD"))
        self.bid_trade.position['BTC'].should.equal(Money(1, "BTC"))

    def test_ask_position(self):
        self.ask_trade.position.should.have.key('USD')
        self.ask_trade.position.should.have.key('BTC')
        self.ask_trade.position['USD'].should.equal(Money(99, "USD"))
        self.ask_trade.position['BTC'].should.equal(Money(-1, "BTC"))

    def test_price_in_currency(self):
        self.bid_trade.price_in_currency("USD").should.equal(Money("100", "USD"))

    def test_cad_price_in_currency(self):
        self.bid_trade.price = Money(100, "CAD")
        self.bid_trade.order.exchange_rate = Decimal("0.8")

        self.bid_trade.price_in_currency("CAD").should.equal(Money("100", "CAD"))
        self.bid_trade.price_in_currency("USD").should.equal(Money("80", "USD"))

    def test_fee_in_currency(self):
        self.bid_trade.fee_in_currency("USD").should.equal(Money("1", "USD"))

    def test_cad_fee_in_currency(self):
        self.bid_trade.fee = Money(1, "CAD")
        self.bid_trade.order.exchange_rate = Decimal("0.8")

        self.bid_trade.fee_in_currency("CAD").should.equal(Money("1", "CAD"))
        self.bid_trade.fee_in_currency("USD").should.equal(Money("0.80", "USD"))

    def test_btc_fee_in_currency(self):
        self.bid_trade.fee = Money("0.01", "BTC")
        self.bid_trade.order.fundamental_value = Money(250, "CAD")
        self.bid_trade.order.exchange_rate = Decimal("0.8")

        self.bid_trade.fee_in_currency("CAD").should.equal(Money("2.5", "CAD"))
        self.bid_trade.fee_in_currency("USD").should.equal(Money("2", "USD"))



# -*- coding: utf-8 -*-
from datetime import datetime, date, timedelta
import json
import os
import uuid

from decimal import *

from gryphon.lib import gryphon_json_serialize
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money


class BasicOrder(object):
    
    #Uncompleted statuses
    OPEN = u'OPEN'
    FILLED = u'FILLED'
    CANCELLED = u'CANCELLED'
    REVERSED = u'REVERSED'

    #Order Types
    BID = Consts.BID
    ASK = Consts.ASK
  
    # Special Actors
    NULL_ACTOR = u'NULL'

    def __unicode__(self):
        return u'[ORDER:%s:%s:%s] Status:%s, Price:%s, Volume:%s BTC' % (
            self.order_type, self.exchange.name, self.currency, self.status, self.price, self.volume)
        
    def __repr__(self):
        return self.to_json()

    #override
    def to_json(self):
        raise NotImplementedError


    #override
    def set_trades(self, trades):
        raise NotImplementedError

    def was_eaten(self, order_details):
        old_position = self.position
        self.time_executed = datetime.utcnow()
        if order_details['btc_total'] <= 0:
            self.status = BasicOrder.CANCELLED
        else:
            self.status = BasicOrder.FILLED
            # TODO store the volume filled amount in order?
            self.time_executed = datetime.utcnow()
            self.set_trades(order_details['trades'])
        return self.position_change(old_position)

    # returns position change
    def was_partially_eaten(self, order_details):
        old_position = self.position
        self.set_trades(order_details['trades'])
        return self.position_change(old_position)

    @property
    def exchange(self):
        if hasattr(self, '_exchange'):
            return self._exchange
        elif self._exchange_name:
            self._exchange = make_exchange_from_key(self._exchange_name)
            return self._exchange
        else:
            return None
        
    @exchange.setter
    def exchange(self, value):
        self._exchange = value
        self._exchange_name = self._exchange.name
    
    
    @property
    def volume(self):
        return Money(self._volume, 'BTC')

    @volume.setter
    def volume(self, value):
        self._volume = value.amount
    
    @property
    def price(self):
        return Money(self._price, 'USD')

    @price.setter
    def price(self, value):
        self._price = value.amount

    #override
    @property
    def volume_filled(self):
        raise NotImplementedError

    @property
    def volume_remaining(self):
        return self.volume - self.volume_filled

    #override
    @property
    def position(self):
        raise NotImplementedError

    def position_change(self, old_position, include_fees=True):
        """
        Returns the difference between an old position and the current position.

        If you are excluding fees, make sure you have also excluded them
        when calculating old_position
        """
        return self.calc_position(include_fees) - old_position



import numpy as np
from cdecimal import Decimal


def ExpMovingAverage(values, timestamps, window_time=300):
    assert len(values) == len(timestamps)
    ema = []
    values = list(values)
    values.reverse()
    timestamps = list(timestamps)
    timestamps.reverse()
    for i in range(len(timestamps)):
        noted_timestamp = timestamps[i]
        relevant_values = []
        relevant_timestamps = []
        for j in range(len(timestamps)):
            
            index = j+i
            if index > len(timestamps)-1:
                break 
            elif noted_timestamp - timestamps[index] < window_time:
                relevant_values.append(values[index])
                relevant_timestamps.append(noted_timestamp - timestamps[index])
            else:
                break
            
        weights = np.exp([ (Decimal(1)/ Decimal(1+ts)) for ts in relevant_timestamps])
        weights = [w/sum(weights) for w in weights]
        
        weighted_value = 0
        for i in range(len(relevant_values)):
            rv = relevant_values[i]
            weight = weights[i]
            weighted_value += rv*weight
        ema.append(weighted_value)
    ema.reverse()
    return ema



"""
Prepared by Gareth MacLeod, 3 April 2018.

This demonstrates that python 2.7 is having series issues with authenticated requests 
to the bitstamp API. This script fails on fresh installs of Ubuntu 14.04 LTS and 16 LTS.

The deepest I've gotten into this bug is this: the requests.Session() object can make
a single successful authenticated request to the API, but then any subsequent
authenticated request fails.

HOWEVER: If you clear the cookies of that session after each request, it then succeeds.
This might sound like a hacky, possibly viable solution, but I suspect that clearing
the cookies actually negates all the performance benefits of keeping a single connection
to bitstamp open.

The python environent is this:

asn1crypto==0.24.0
certifi==2018.1.18
cffi==1.11.5
chardet==3.0.4
click==6.7
cryptography==2.2.2
enum34==1.1.6
idna==2.6
ipaddress==1.0.19
ordereddict==1.1
pycparser==2.18
pyOpenSSL==17.5.0
python-dotenv==0.3.0
requests==2.18.4
six==1.11.0
urllib3==1.22

OpenSSL version 1.0.2g.

You can construct this environment just by setting up a fresh Ubuntu 14/16 install, then
running

sudo apt update
sudo apt install python python-pip
pip install requests[security]

Make sure you have yoru bitstamp credentials in a .env file in the directory you are
running from.
"""

import requests
import hmac
import time
import hashlib
import time

import dotenv
import os
import os.path

dotenv_path = os.path.join(os.path.dirname(__file__), '.env')
dotenv.load_dotenv(dotenv_path)


API_KEY = str(os.environ['BITSTAMP_ETH_EUR_API_KEY'])
SECRET = str(os.environ['BITSTAMP_ETH_EUR_API_SECRET'])
CLIENT_ID = str(os.environ['BITSTAMP_ETH_EUR_CLIENT_ID'])

API_BALANCE_URL = 'https://www.bitstamp.net/api/v2/balance/'


def requests_dot_post():
    print '---'
    print "Trying requests.post"

    nonce = unicode(int(round(time.time() * 1000)))
    message = nonce + CLIENT_ID + API_KEY
    sig = hmac.new(SECRET, msg=message, digestmod=hashlib.sha256).hexdigest().upper()

    payload = {
        'nonce': nonce,
        'key': API_KEY,
        'signature': sig,
    }

    result = requests.post(API_BALANCE_URL, data=payload).text

    print 'Response from bitstamp: %s' % result[:50]

    try:
        assert('btc_available' in result)
        print 'requests.post succeeded!'
    except Exception as e:
        print 'requests.post failed: %s' % str(e)


def session_post(session=None, clear_cookies=None):
    print '---'

    if session is None:
        print "Trying with a new session"

        session = requests.Session()
    else:
        print "Trying with an extant session"

        if clear_cookies is True:
            print "Clearing the session's cookies"
            session.cookies.clear()

    nonce = unicode(int(round(time.time() * 1000)))
    message = nonce + CLIENT_ID + API_KEY
    sig = hmac.new(SECRET, msg=message, digestmod=hashlib.sha256).hexdigest().upper()

    payload = {
        'nonce': nonce,
        'key': API_KEY,
        'signature': sig,
    }

    result = session.post(API_BALANCE_URL, payload).text

    print 'Response from bitstamp: %s' % result[:50]

    try:
        assert('btc_available' in result)
        print 'Session succeeded!'
    except Exception as e:
        print 'Session failed: %s' % str(e)

    return session


requests_dot_post()
requests_dot_post()
session = session_post()
session_post(session=session)
session_post(session=session, clear_cookies=True)





from kraken_orderbook_poller import KrakenOrderbook


class KrakenCADOrderbook(KrakenOrderbook):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_CAD'
        self.currency = 'CAD'
        self.poll_time = 2
        self.orderbook_depth = 100000



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.open_exchange_rate_poller import OpenExchangeRatePoller
from gryphon.data_service.task import Task


class UtilitiesPollTask(Task):
    def __init__(self):
        self.pollers = [
            OpenExchangeRatePoller()
        ]

    def start_task(self):
        for poller in self.pollers:
            poller.start()



"""
Functions we use to record tick time performance stats.
"""

from delorean import Delorean

from gryphon.lib.models.datum import Datum, DatumRecorder
from gryphon.lib.util.profile import tick_profile, tick_profile_data


TICK_SAMPLE_SIZE = 10
TICK_BLOCK_SAMPLE_SIZE = 100


def record_tick_data(tick_start, strategy_name):
    tick_end = Delorean().epoch
    tick_length = tick_end - tick_start

    datum_name = '%s_TICK_TIME' % strategy_name
    DatumRecorder().record_mean(datum_name, tick_length, TICK_SAMPLE_SIZE)


def record_tick_block_data(algo, tick_count, strategy_name):
    for function_name, profile_times in tick_profile_data.iteritems():
        datum_name = datum_name_for_function_block(strategy_name, function_name)

        for block_time in profile_times:
            DatumRecorder().record_mean(datum_name, block_time, TICK_BLOCK_SAMPLE_SIZE)

        tick_profile_data[function_name] = []


def datum_name_for_function_block(strategy_name, function_name):
    datum_name = '%s_TICK_BLOCK_TIME_%s' % (
        strategy_name.upper(),
        function_name.upper(),
    )

    return datum_name




import logging
def log_request_perf(handler):
    
    if handler.get_status() < 400:
        log_method = logging.info
    elif handler.get_status() < 500:
        log_method = logging.warning
    else:
        log_method = logging.error
    request_time = 1000.0 * handler.request.request_time()
    request_speed_message=''
    if request_time <= 100:
        request_speed_message = 'FAST'
    elif request_time <= 200:
        request_speed_message = 'SPEED OK'
    elif request_time <= 400:
        request_speed_message= 'PRETTY SLOW'
    elif request_time <= 600:
        request_speed_message= 'SLOW'
    elif request_time <= 800:
        request_speed_message= 'VERY SLOW'
    elif request_time > 800:
        request_speed_message= 'CRITICALLY SLOW'
           
    log_method("%d %s %.2fms - %s", handler.get_status(),
               handler._request_summary(), request_time, request_speed_message)



import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import os
import unittest
import sure
import mock
from mock import patch
from decimal import Decimal

from gryphon.lib.money import Money
from gryphon.lib.test_helper import *

import logging
logger = logging.getLogger(__name__)

# the money model we base ours on (https://github.com/carlospalol/money)
# has good tests, so we only need to test the specific features we have added

class TestMoney():
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_invalid_currency(self):
        Money.when.called_with("100", "XXX").should.throw(ValueError)

    def test_currency_conversion_to_same_currency(self):
        m = Money(100, "USD")
        m.to("USD").should.equal(m)

    # we have to be careful here to not cache bad data in memcache
    @patch('gryphon.lib.forex.USDCurrencyConverter')
    def test_currency_conversion(self, patched_currency_converter):
        patched_currency_converter._all_rates = mock.Mock(return_value={'USD': 1, 'CAD': Decimal("2")})

        m = Money(100, "CAD")
        expected_usd = Money(50, "USD")
        m.to("USD").should.equal(expected_usd)

    def test_currency_conversion_passing_exchange_rate(self):
        m = Money(100, "CAD")
        rate_to_usd = Decimal("0.8") # ouch
        expected_usd = Money(80, "USD")
        m.to("USD", exchange_rate_to_usd=rate_to_usd).should.equal(expected_usd)

    def test_thousands_commas_in_numbers(self):
        m_as_int = Money(1000, "USD")

        m_with_commas = Money("1,000", "USD")
        m_with_commas.should.equal(m_as_int)

        m_with_commas_loads = Money.loads("USD 1,000")
        m_with_commas_loads.should.equal(m_as_int)

    def test_round_to_decimal_places(self):
        m = Money('5.12345678', 'BTC')
        rounded_to_4 = Money('5.1235', 'BTC')
        rounded_to_2 = Money('5.13', 'BTC')

        m.round_to_decimal_places(4).should.equal(rounded_to_4)
        m.round_to_decimal_places(2).should.equal(rounded_to_2)
        m.round_to_decimal_places(8).should.equal(m)

    def test_round_to_bucket(self):
        m = Money('3223.45', 'USD')
        rounded_to_100 = Money('3200', 'USD')
        rounded_to_1000 = Money('3000', 'USD')

        m.round_to_bucket(100).should.equal(rounded_to_100)
        m.round_to_bucket(1000).should.equal(rounded_to_1000)
        m.round_to_bucket(Decimal('0.01')).should.equal(m)



from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class GeminiETHUSDOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'GEMINI_ETH_USD'
        self.url = 'https://api.gemini.com/v1/book/ETHUSD?limit_bids=0&limit_asks=0'
        self.poll_time = 5  # Unclear what this should be.

    # API response format:
    # {"bids":[{"price":"405.02","amount":"0.001","timestamp":"1411774902.0"}, ...

    def parse_order(self, order):
        return [order['price'], order['amount'], order['timestamp']]



from twisted.python import log

from gryphon.data_service.pollers.volume.volume_poller import VolumePoller


class BitfinexVolume(VolumePoller):
    """
    This endpoint currently has a rate limit of 30/minute.
    """

    def __init__(self):
        self.exchange_name = u'BITFINEX_BTC_USD'
        self.url = 'https://api.bitfinex.com/v1/pubticker/BTCUSD'

    def parse_volume_json(self, volume_dict):
        try:
            return volume_dict['volume']
        except Exception as e:
            log.msg('Error getting Bitfinex Volume %s' % e)



from collections import defaultdict
from datetime import timedelta
import json

from delorean import Delorean
from sqlalchemy import and_
import treq
from twisted.internet import defer
from twisted.internet import reactor
from twisted.internet.task import LoopingCall
from twisted.python import log
from twisted.python.filepath import FilePath

from gryphon.data_service.auditors.auditor import Auditor
import gryphon.data_service.util as util
from gryphon.lib.dict_differ import DictDiffer
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.exchange.base import Exchange
from gryphon.lib.metrics import midpoint
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook
from gryphon.lib.money import Money

metadata = EmeraldHavocBase.metadata
orderbook_table = metadata.tables['orderbook']

# This determines how deep we audit the orderbook, necessary for performance reasons.
ORDERBOOK_PRICE_LIMIT = 10


class OrderbookAuditor(Auditor):
    def __init__(self):
        self.unsuccessful_audit_threshold = 4
        self.incomplete_audit_threshold = 60
        self.acceptable_fund_value_threshold = 0.0025
        self.acceptable_changes_threshold = 10
        self.audit_time = 10

    @defer.inlineCallbacks
    def start(self):
        self.setup_mysql()
        self.orderbook_timestamp = Delorean().datetime
        self.unsuccessful_audits = 0
        self.incomplete_audits = 0
        self.touch_path = 'monit/heartbeat/%s_orderbook.txt' % self.exchange_name.lower()
        self.looping_call = LoopingCall(self.pre_audit)
        self.looping_call.start(self.audit_time)
        self.redis = yield util.setup_redis()
        self.should_continue_key = '%s_orderbook_should_continue' % self.exchange_name.lower()
        self.redis.set(self.should_continue_key, 1)

    @defer.inlineCallbacks
    def pre_audit(self):
        if self.should_hard_fail():
            self.hard_fail()
            return

        try:
            # Coinbase requires a user agent
            headers = {'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:32.0) Gecko/20100101 Firefox/32.0'}
            response = yield treq.get(self.orderbook_url, headers=headers)
            new_orderbook = yield treq.json_content(response)
        except Exception as e:
            log.msg('%s Orderbook Auditor: %s' % (self.exchange_name, str(e)))

            self.incomplete_audits += 1

            log.msg(
                '%s Incomplete Audits: %s' % (
                self.exchange_name,
                self.incomplete_audits,
            ))

            return

        new_orderbook_timestamp = self.get_timestamp(new_orderbook)

        if new_orderbook_timestamp <= self.orderbook_timestamp:
            # We already audited this orderbook.
            return

        self.orderbook_timestamp = new_orderbook_timestamp

        # Found orderbook. Auditing it 10s from now to make sure we have websocket
        # orderbooks in the -5s +5s db window we look up.
        reactor.callLater(10, self.audit, new_orderbook, new_orderbook_timestamp)

    @defer.inlineCallbacks
    def audit(self, orderbook, orderbook_timestamp):
        audit_successful = yield self.audit_orderbook(orderbook, orderbook_timestamp)

        if audit_successful == 'SOFT':
            self.unsuccessful_audits += 1

            if self.should_hard_fail():
                self.hard_fail()
            else:
                log.msg(
                    '%s Orderbook Audit Soft Failed: %s' % (
                    self.exchange_name,
                    self.unsuccessful_audits,
                ))
        elif audit_successful == 'HARD':
            self.hard_fail()
        else:
            FilePath(self.touch_path).touch()
            self.unsuccessful_audits = 0
            self.incomplete_audits = 0

            log.msg(
                '%s Orderbook Audit Passed on Timestamp:%s' % (
                self.exchange_name,
                orderbook_timestamp,
            ))

    def should_hard_fail(self):
        too_many_unsuccessful_audits = (
            self.unsuccessful_audits > self.unsuccessful_audit_threshold
        )

        too_many_incomplete_audits = (
            self.incomplete_audits > self.incomplete_audit_threshold
        )

        return too_many_unsuccessful_audits or too_many_incomplete_audits

    @defer.inlineCallbacks
    def hard_fail(self):
        log.err('%s Orderbook Auditor Declaring a Hard Failure' % self.exchange_name)

        yield self.redis.set(self.should_continue_key, 0)
        self.looping_call.stop()

    @defer.inlineCallbacks
    def audit_orderbook(self, orderbook, orderbook_timestamp):
        orderbook_timestamp_early = orderbook_timestamp - timedelta(seconds=5)
        orderbook_timestamp_late = orderbook_timestamp + timedelta(seconds=5)

        result = yield self.engine.execute(
            orderbook_table.select(orderbook_table).where(
                and_(
                    orderbook_table.c.exchange == self.exchange_name,
                    orderbook_table.c.timestamp.between(
                        orderbook_timestamp_early,
                        orderbook_timestamp_late))))

        our_orderbooks = yield result.fetchall()

        # Non Blocking ^^^
        # Potentially Blocking vvv

        start_time = Delorean().epoch
        audit_successful = 'SOFT'
        change_dict = {}
        fundamental_values = {}

        exchange_object = exchange_factory.make_exchange_from_key(self.exchange_name)

        price_limit = Money(ORDERBOOK_PRICE_LIMIT, exchange_object.currency)

        http_orderbook = exchange_object.parse_orderbook(
            orderbook,
            price_limit=price_limit,
        )

        http_fundamental_value = self.fundamental_value(http_orderbook)
        indexed_http_ob = self.index_orderbook(http_orderbook)

        for our_ob in our_orderbooks:
            raw_db_orderbook = {
                'bids': json.loads(our_ob.bids),
                'asks': json.loads(our_ob.asks),
            }

            db_orderbook = exchange_object.parse_orderbook(
                raw_db_orderbook,
                price_limit=price_limit,
                cached_orders=True,
            )

            # Check for soft falilures
            db_fundamental_value = self.fundamental_value(db_orderbook)

            fund_value_closeness = (
                abs(db_fundamental_value - http_fundamental_value)
                / http_fundamental_value
            )

            indexed_db_ob = self.index_orderbook(db_orderbook)

            ask_diffs = DictDiffer(indexed_db_ob['asks'], indexed_http_ob['asks'])
            bid_diffs = DictDiffer(indexed_db_ob['bids'], indexed_http_ob['bids'])

            changes = (
                list(ask_diffs.added()) +
                list(ask_diffs.removed()) +
                list(ask_diffs.changed()) +
                list(bid_diffs.added()) +
                list(bid_diffs.removed()) +
                list(bid_diffs.changed())
            )

            total_changes = len(changes)

            change_dict[total_changes] = changes
            fundamental_values[fund_value_closeness] = {
                'db_fundamental_value': db_fundamental_value,
                'http_fundamental_value': http_fundamental_value,
            }

            hard_failure_fund_value_closeness = 10

            if (total_changes < self.acceptable_changes_threshold and
                    fund_value_closeness < self.acceptable_fund_value_threshold):
                audit_successful = 'SUCCESSFUL'

            # Check for hard failures.
            if self.detect_orderbook_cross(db_orderbook):
                audit_successful = 'HARD'

                log.err('%s Auditor Detects Cross -  Bids:%s, Asks:%s' % (
                    self.exchange_name,
                    db_orderbook['bids'][:3],
                    db_orderbook['asks'][:3]
                ))

                break

            if fund_value_closeness > hard_failure_fund_value_closeness:
                audit_successful = 'HARD'

                log.err('Funamental Value difference is more than %s% its:' % (
                    hard_failure_fund_value_closeness,
                    fund_value_closeness
                ))

                break

        if not audit_successful == 'SUCCESSFUL':
            log.msg('%s Orderbook Auditor Soft Failure Report:' % self.exchange_name)

            if not our_orderbooks:
                log.msg('No orderbooks to audit against')

            for key, value in fundamental_values.iteritems():
                log.msg(
                    '------ Fundamental Value Closeness:%.6f, DBfv:%s, HTTPfv:%s' % (
                    key,
                    value['db_fundamental_value'],
                    value['http_fundamental_value']
                ))

            for key, value in change_dict.iteritems():
                log.msg('------ Change Count: %s' % key)

        log.msg(
            'Time Elapsed Auditing %s Orderbook: %s' % (
            self.exchange_name,
            Delorean().epoch - start_time,
        ))

        defer.returnValue(audit_successful)

    def index_orderbook(self, orderbook):
        """Returns bid and ask dictionary of price => volume."""

        indexed_orderbook = {}
        indexed_orderbook['bids'] = self.index_orders(orderbook['bids'])
        indexed_orderbook['asks'] = self.index_orders(orderbook['asks'])

        return indexed_orderbook

    def index_orders(self, orders):
        """Returns a dictionary of price => volume."""

        orders_dict = defaultdict(int)

        for order in orders:
            orders_dict[order.price.amount] += order.volume.amount

        return orders_dict

    def fundamental_value(self, orderbook):
        depth = Money('20', 'BTC')

        try:
            fundamental_value = midpoint.get_midpoint_from_orderbook(orderbook, depth)
        except:
            # Some very illiquid exchanges don't always have 20 BTC of depth.
            depth = Money('1', 'BTC')

            fundamental_value = midpoint.get_midpoint_from_orderbook(orderbook, depth)

        return fundamental_value

    def detect_orderbook_cross(self, orderbook):
        if orderbook['asks'] and orderbook['bids']:
            if orderbook['asks'][0].price <= orderbook['bids'][0].price:
                return True

        return False



"""
These function calculate the volume available to buy/sell at a given price point in an
orderbook. Another phrase for this would be "orderbook levels".
"""

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts


def volume_available_at_price(mode, price, orderbook):
    """
    This uses bid/ask to represent intention, not the side of the order book to look
    at. volume_avail_at_price(ASK, 250) will give you how many bitcoins can be sold
    for at least $250 in the orderbook.
    """
    if(mode == Consts.BID):
        orders = orderbook.get('asks', [])
    elif(mode == Consts.ASK):
        orders = orderbook.get('bids', [])
    else:
        raise ValueError('mode must be one of ask/bid')

    if not orders:
        raise Exception('no orders on one side of the book.')

    volume_available = Money('0', 'BTC')

    for order in orders:
        if mode == Consts.BID:
            if order.price <= price:
                volume_available += order.volume
            else:
                break
        elif mode == Consts.ASK:
            if order.price >= price:
                volume_available += order.volume
            else:
                break

    return volume_available


def volume_available_at_prices(mode, prices, orderbook):
    """
    Same as above but gets the volume available at multiple levels at once. This
    gives us a substantial performance bump which is necessary in the importer.

    Prices should be money object created by str'ing a float.
    Returns a decimal: Money dict. It would be more consistent with the singular
    level function to return a Money: Money, but money is unhashable.
    """

    if mode == Consts.BID:
        orders = orderbook.get('asks', [])
    elif mode == Consts.ASK:
        orders = orderbook.get('bids', [])
    else:
        raise ValueError('mode must be one of ask/bid')

    if not orders:
        raise Exception('no orders on one side of the book.')

    volume_available = Money('0', 'BTC')
    levels = {}
    i = 0
    j = 0

    # Loop until one of the two lists is exhausted.
    if mode == Consts.BID:
        prices = sorted(prices, reverse=False)

        while True:
            if orders[i].price <= prices[j]:
                volume_available += orders[i].volume
                i = i + 1
            elif orders[i].price > prices[j]:
                levels[prices[j].amount] = volume_available
                j = j + 1

            if i == len(orders) or j == len(prices):
                break

    elif mode == Consts.ASK:
        prices = sorted(prices, reverse=True)

        while True:
            if orders[i].price >= prices[j]:
                volume_available += orders[i].volume
                i = i + 1
            elif orders[i].price < prices[j]:
                levels[prices[j].amount] = volume_available
                j = j + 1

            if i == len(orders) or j == len(prices):
                break

    # If we broke because we ran out of orders, fill in the rest of the dict.
    while j < len(prices):
        levels[prices[j].amount] = volume_available
        j += 1

    return levels




# -*- coding: utf-8 -*-
from datetime import timedelta

from delorean import Delorean
from sqlalchemy import and_
from twisted.internet import defer
from twisted.internet import reactor
from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import LoopingCall
from twisted.python import log

from gryphon.data_service.auditors.auditor import Auditor
import gryphon.data_service.util as util
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
from gryphon.lib.twistedbitcoinwisdom import TwistedBitcoinWisdom

metadata = EmeraldHavocBase.metadata
trades = metadata.tables['trade']


EXCHANGES = ['KRAKEN', 'BITSTAMP', 'BITFINEX', 'CAVIRTEX', 'VAULTOFSATOSHI']


class TradesVolumeAuditor(Auditor):
    def start(self):
        self.redis = yield util.setup_redis()
        self.setup_mysql()
        self.looping_call = LoopingCall(self.audit).start(30)

    @defer.inlineCallbacks
    def target_volume_in_trade_list(self, trade_list, target_volume):
        n = len(trade_list)
        found_match = False

        def offline_summation(lower_bound, upper_bound, trade_list, trades, target_volume):
            perm_sum = 0

            for k in range(lower_bound, upper_bound):
                perm_sum += trade_list[k][trades.c.volume]

            return perm_sum == target_volume

        if not n:
            defer.returnValue(target_volume == 0)
        elif target_volume == 0:
            # TODO Fix this.
            defer.returnValue(False)
        else:
            for i in range(n):
                for j in range(n - i, n + 1):
                    found_match = yield reactor.callLater(
                        0,
                        offline_summation,
                        j - (n - i),
                        j,
                        trade_list,
                        trades,
                        target_volume,
                    )

                    if found_match:
                        break

            defer.returnValue(found_match)

    @inlineCallbacks
    def audit(self, exchanges=EXCHANGES):
        now = Delorean().datetime
        outer_min = now
        outer_max = Delorean(now - timedelta(minutes=9)).datetime
        inner_max = Delorean(now - timedelta(minutes=6)).datetime
        inner_min = Delorean(now - timedelta(minutes=3)).datetime

        for exchange in exchanges:
            real_volume = yield TwistedBitcoinWisdom(exchange=exchange)\
                .volume_in_period(inner_max, inner_min)

            real_volume = real_volume.amount

            result = yield self.engine.execute(
                trades.select(trades)
                .where(and_(
                    trades.c.exchange.startswith(exchange),
                    trades.c.timestamp.between(outer_max, outer_min),
                ))
            )

            d_trades = yield result.fetchall()

            target_in_trades = yield self.target_volume_in_trade_list(
                d_trades,
                real_volume,
            )

            if not target_in_trades:
                log.err('Trade Volume Audit Failed : %s' % (exchange))
            else:
                log.msg('Trade Volume Audit Passed : %s' % (exchange))




# -*- coding: utf-8 -*-
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
import json
import uuid
from decimal import *

from gryphon.lib.money import Money
from datetime import datetime
from sqlalchemy import Column, Integer, Unicode, DateTime, Numeric
from sqlalchemy.ext.declarative import declarative_base

metadata = EmeraldHavocBase.metadata


class ExchangeVolume(EmeraldHavocBase):
    __tablename__ = 'exchange_volume'

    exchange_volume_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    exchange = Column(Unicode(64), nullable=False)
    timestamp = Column(DateTime, nullable=False)
    time_created = Column(DateTime, nullable=False)
    _exchange_volume = Column('exchange_volume', Numeric(precision=20, scale=10))

    def __init__(self, exchange_volume, exchange, timestamp):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.timestamp = timestamp
        self.exchange_volume = exchange_volume
        self.exchange = exchange

    def __unicode__(self):
        return u'[EXCHANGE VOLUME - Exchange:%s] Volume:%s,' % (
            self.exchange, self.exchange_volume)

    def __repr__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps({
            'exchange_volume_id': self.exchange_volume_id,
            'time_created': unicode(self.time_created),
            'unique_id': self.unique_id,
            'exchange': self.exchange,
            'exchange_volume': self.exchange_volume
        }, ensure_ascii=False)

    @property
    def exchange_volume(self):
        return Money(self._exchange_volume, 'BTC')

    @exchange_volume.setter
    def exchange_volume(self, value):
        if isinstance(value, Money) and value.currency == 'BTC':
            self._exchange_volume = value.amount
        else:
            raise ValueError('exchange_volume must be a BTC Money object')
